@@ -14,6 +14,7 @@
 #include "nvstatus.h"
 #include "nv-misc.h"
 #include "nv.h"
+#include "nv-ioctl-numa.h"
 #include "conftest.h"
 
 #include "nv-lock.h"
@@ -116,7 +117,6 @@
 
 #include <linux/moduleparam.h>      /* module_param()                   */
 #include <asm/tlbflush.h>           /* flush_tlb(), flush_tlb_all()     */
-#include <asm/kmap_types.h>         /* page table entry lookup          */
 
 #include <linux/pci.h>              /* pci_find_class, etc              */
 #include <linux/interrupt.h>        /* tasklets, interrupt helpers      */
@@ -126,6 +126,7 @@
 #include <linux/cpu.h>              /* CPU hotplug support              */
 
 #include <linux/pm_runtime.h>       /* pm_runtime_*                     */
+#include <linux/fdtable.h>          /* files_fdtable, etc               */
 
 
 
@@ -750,15 +751,11 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
 #define NV_PCI_DEVFN(pci_dev)         (pci_dev)->devfn
 #define NV_PCI_SLOT_NUMBER(pci_dev)   PCI_SLOT(NV_PCI_DEVFN(pci_dev))
 
-#define NV_PCI_DEV_PUT(pci_dev)                pci_dev_put(pci_dev)
-#define NV_PCI_GET_DEVICE(vendor,device,from)  pci_get_device(vendor,device,from)
-#define NV_PCI_GET_CLASS(class,from)           pci_get_class(class,from)
-
 #if defined(CONFIG_X86_UV) && defined(NV_CONFIG_X86_UV)
 #define NV_GET_DOMAIN_BUS_AND_SLOT(domain,bus,devfn)                        \
    ({                                                                       \
         struct pci_dev *__dev = NULL;                                       \
-        while ((__dev = NV_PCI_GET_DEVICE(PCI_VENDOR_ID_NVIDIA,             \
+        while ((__dev = pci_get_device(PCI_VENDOR_ID_NVIDIA,                \
                     PCI_ANY_ID, __dev)) != NULL)                            \
         {                                                                   \
             if ((NV_PCI_DOMAIN_NUMBER(__dev) == domain) &&                  \
@@ -770,7 +767,7 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
         }                                                                   \
         if (__dev == NULL)                                                  \
         {                                                                   \
-            while ((__dev = NV_PCI_GET_CLASS((PCI_CLASS_BRIDGE_HOST << 8),  \
+            while ((__dev = pci_get_class((PCI_CLASS_BRIDGE_HOST << 8),     \
                         __dev)) != NULL)                                    \
             {                                                               \
                 if ((NV_PCI_DOMAIN_NUMBER(__dev) == domain) &&              \
@@ -783,7 +780,7 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
         }                                                                   \
         if (__dev == NULL)                                                  \
         {                                                                   \
-            while ((__dev = NV_PCI_GET_CLASS((PCI_CLASS_BRIDGE_PCI << 8),   \
+            while ((__dev = pci_get_class((PCI_CLASS_BRIDGE_PCI << 8),      \
                         __dev)) != NULL)                                    \
             {                                                               \
                 if ((NV_PCI_DOMAIN_NUMBER(__dev) == domain) &&              \
@@ -796,7 +793,7 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
         }                                                                   \
         if (__dev == NULL)                                                  \
         {                                                                   \
-            while ((__dev = NV_PCI_GET_DEVICE(PCI_ANY_ID, PCI_ANY_ID,       \
+            while ((__dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID,          \
                             __dev)) != NULL)                                \
             {                                                               \
                 if ((NV_PCI_DOMAIN_NUMBER(__dev) == domain) &&              \
@@ -816,7 +813,7 @@ static inline dma_addr_t nv_phys_to_dma(struct device *dev, NvU64 pa)
 #define NV_GET_DOMAIN_BUS_AND_SLOT(domain,bus,devfn)               \
    ({                                                              \
         struct pci_dev *__dev = NULL;                              \
-        while ((__dev = NV_PCI_GET_DEVICE(PCI_ANY_ID, PCI_ANY_ID,  \
+        while ((__dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID,     \
                     __dev)) != NULL)                               \
         {                                                          \
             if ((NV_PCI_DOMAIN_NUMBER(__dev) == domain) &&         \
@@ -1539,6 +1536,12 @@ typedef struct nv_linux_state_s {
 
 
 
+
+
+
+
+
+
     /* Lock serializing ISRs for different SOC vectors */
     nv_spinlock_t soc_isr_lock;
 
@@ -1555,6 +1558,7 @@ typedef struct nv_linux_state_s {
     NvBool is_forced_shutdown;
 
     struct nv_dma_device dma_dev;
+    struct nv_dma_device niso_dma_dev;
 } nv_linux_state_t;
 
 extern nv_linux_state_t *nv_linux_devices;
@@ -1735,16 +1739,9 @@ static inline NV_STATUS nv_check_gpu_state(nv_state_t *nv)
 extern NvU32 NVreg_EnableUserNUMAManagement;
 extern NvU32 NVreg_RegisterPCIDriver;
 
-extern nv_pci_info_t nv_assign_gpu_pci_info[NV_MAX_DEVICES];
-extern NvU32 nv_assign_gpu_count;
 extern NvU32 num_probed_nv_devices;
 extern NvU32 num_nv_devices;
 
-#define NV_IS_ASSIGN_GPU_PCI_INFO_SPECIFIED()     \
-    (!((nv_assign_gpu_pci_info[0].domain == 0) && \
-       (nv_assign_gpu_pci_info[0].bus == 0) &&    \
-       (nv_assign_gpu_pci_info[0].slot == 0)))
-
 #if defined(NV_FILE_HAS_INODE)
 #define NV_FILE_INODE(file) (file)->f_inode
 #else
@@ -1922,6 +1919,10 @@ static inline NvU32 nv_default_irq_flags(nv_state_t *nv)
 
 NvS32 nv_request_soc_irq(nv_linux_state_t *, NvU32, nv_soc_irq_type_t, NvU32, NvU32);
 
+
+
+
+
 static inline void nv_mutex_destroy(struct mutex *lock)
 {
 
