@@ -24,6 +24,8 @@
 #include <nvCpuUuid.h>
 #include <stdarg.h>
 #include <nv-caps.h>
+#include <nv-ioctl.h>
+#include <nvmisc.h>
 
 extern nv_cap_t *nvidia_caps_root;
 
@@ -36,51 +38,17 @@ extern nv_cap_t *nvidia_caps_root;
 #include <nv-kernel-interface-api.h>
 #endif
 
-#if !defined(NV_MIN)
-#define NV_MIN(_a,_b) ((_a) < (_b) ? (_a) : (_b))
-#endif
-#if !defined(NV_MAX)
-#define NV_MAX(_a,_b) ((_a) > (_b) ? (_a) : (_b))
-#endif
-
 /* NVIDIA's reserved major character device number (Linux). */
 #define NV_MAJOR_DEVICE_NUMBER 195
 
 #define GPU_UUID_LEN    (16)
 
-#define NV_MAX_RM_EXPORTED_OBJECTS (2)
-
 /*
  * Buffer size for an ASCII UUID: We need 2 digits per byte, plus space
  * for "GPU", 5 dashes, and '\0' termination:
  */
 #define GPU_UUID_ASCII_LEN  (GPU_UUID_LEN * 2 + 9)
 
-typedef struct {
-    NvU32    domain;        /* PCI domain number   */
-    NvU8     bus;           /* PCI bus number      */
-    NvU8     slot;          /* PCI slot number     */
-    NvU8     function;      /* PCI function number */
-    NvU16    vendor_id;     /* PCI vendor ID       */
-    NvU16    device_id;     /* PCI device ID       */
-    NvBool   valid;         /* validation flag     */
-} nv_pci_info_t;
-
-/* NOTE: using an ioctl() number > 55 will overflow! */
-#define NV_IOCTL_MAGIC      'F'
-#define NV_IOCTL_BASE       200
-#define NV_ESC_CARD_INFO         (NV_IOCTL_BASE + 0)
-#define NV_ESC_ALLOC_OS_EVENT    (NV_IOCTL_BASE + 6)
-#define NV_ESC_FREE_OS_EVENT     (NV_IOCTL_BASE + 7)
-#define NV_ESC_STATUS_CODE       (NV_IOCTL_BASE + 9)
-#define NV_ESC_CHECK_VERSION_STR (NV_IOCTL_BASE + 10)
-#define NV_ESC_IOCTL_XFER_CMD    (NV_IOCTL_BASE + 11)
-#define NV_ESC_ATTACH_GPUS_TO_FD (NV_IOCTL_BASE + 12)
-#define NV_ESC_QUERY_DEVICE_INTR (NV_IOCTL_BASE + 13)
-#define NV_ESC_SYS_PARAMS        (NV_IOCTL_BASE + 14)
-#define NV_ESC_NUMA_INFO         (NV_IOCTL_BASE + 15)
-#define NV_ESC_SET_NUMA_STATUS   (NV_IOCTL_BASE + 16)
-
 /*
  * #define an absolute maximum used as a sanity check for the
  * NV_ESC_IOCTL_XFER_CMD ioctl() size argument.
@@ -98,170 +66,12 @@ typedef struct {
     _IOWR(NV_IOCTL_MAGIC, (nr), type);                                \
 })
 
-/*
- * ioctl()'s with parameter structures too large for the
- * _IOC cmd layout use the nv_ioctl_xfer_t structure
- * and the NV_ESC_IOCTL_XFER_CMD ioctl() to pass the actual
- * size and user argument pointer into the RM, which
- * will then copy it to/from kernel space in separate steps.
- */
-typedef struct nv_ioctl_xfer
-{
-    NvU32   cmd;
-    NvU32   size;
-    NvP64   ptr  NV_ALIGN_BYTES(8);
-} nv_ioctl_xfer_t;
-
-typedef struct nv_ioctl_card_info
-{
-    NvU16         flags;               /* see below                   */
-    nv_pci_info_t pci_info;            /* PCI config information      */
-    NvU32         gpu_id;
-    NvU16         interrupt_line;
-    NvU64         reg_address    NV_ALIGN_BYTES(8);
-    NvU64         reg_size       NV_ALIGN_BYTES(8);
-    NvU64         fb_address     NV_ALIGN_BYTES(8);
-    NvU64         fb_size        NV_ALIGN_BYTES(8);
-    NvU32         minor_number;
-    NvU8          dev_name[10];  /* device names such as vmgfx[0-32] for vmkernel */
-} nv_ioctl_card_info_t;
-
-#define NV_IOCTL_CARD_INFO_BUS_TYPE_PCI            0x0001
-#define NV_IOCTL_CARD_INFO_BUS_TYPE_AGP            0x0002
-#define NV_IOCTL_CARD_INFO_BUS_TYPE_PCI_EXPRESS    0x0003
-
-#define NV_IOCTL_CARD_INFO_FLAG_PRESENT       0x0001
-
-#define SIM_ENV_GPU       0
-#define SIM_ENV_IKOS      1
-#define SIM_ENV_CSIM      2
-
-#define NV_SLI_DISABLED   0
-#define NV_SLI_ENABLED    1
-
-/* old rm api check
- *
- * this used to be used to verify client/rm interaction both ways by
- * overloading the structure passed into the NV_IOCTL_CARD_INFO ioctl.
- * This interface is deprecated and NV_IOCTL_CHECK_VERSION_STR should
- * be used instead.  We keep the structure and defines here so that RM
- * can recognize and handle old clients.
- */
-typedef struct nv_ioctl_rm_api_old_version
-{
-    NvU32 magic;
-    NvU32 major;
-    NvU32 minor;
-    NvU32 patch;
-} nv_ioctl_rm_api_old_version_t;
-
-#define NV_RM_API_OLD_VERSION_MAGIC_REQ              0x0197fade
-#define NV_RM_API_OLD_VERSION_MAGIC_REP              0xbead2929
-#define NV_RM_API_OLD_VERSION_MAGIC_LAX_REQ         (NV_RM_API_OLD_VERSION_MAGIC_REQ ^ '1')
-#define NV_RM_API_OLD_VERSION_MAGIC_OVERRIDE_REQ    (NV_RM_API_OLD_VERSION_MAGIC_REQ ^ '2')
-#define NV_RM_API_OLD_VERSION_MAGIC_IGNORE           0xffffffff
-
-typedef enum {
-    NV_CPU_TYPE_UNKNOWN = 0,
-    NV_CPU_TYPE_ARM_A9
-} nv_cpu_type_t;
-
-/* alloc event */
-typedef struct nv_ioctl_alloc_os_event
-{
-    NvHandle hClient;
-    NvHandle hDevice;
-    NvU32    fd;
-    NvU32    Status;
-} nv_ioctl_alloc_os_event_t;
-
-/* free event */
-typedef struct nv_ioctl_free_os_event
-{
-    NvHandle hClient;
-    NvHandle hDevice;
-    NvU32    fd;
-    NvU32    Status;
-} nv_ioctl_free_os_event_t;
-
 #define NV_PCI_DEV_FMT          "%04x:%02x:%02x.%x"
 #define NV_PCI_DEV_FMT_ARGS(nv) (nv)->pci_info.domain, (nv)->pci_info.bus, \
                                 (nv)->pci_info.slot, (nv)->pci_info.function
 
-/* status code */
-typedef struct nv_ioctl_status_code
-{
-    NvU32 domain;
-    NvU8  bus;
-    NvU8  slot;
-    NvU32 status;
-} nv_ioctl_status_code_t;
-
-/* check version string */
-#define NV_RM_API_VERSION_STRING_LENGTH 64
-
-typedef struct nv_ioctl_rm_api_version
-{
-    NvU32 cmd;
-    NvU32 reply;
-    char versionString[NV_RM_API_VERSION_STRING_LENGTH];
-} nv_ioctl_rm_api_version_t;
-
-#define NV_RM_API_VERSION_CMD_STRICT         0
-#define NV_RM_API_VERSION_CMD_RELAXED       '1'
-#define NV_RM_API_VERSION_CMD_OVERRIDE      '2'
-
-#define NV_RM_API_VERSION_REPLY_UNRECOGNIZED 0
-#define NV_RM_API_VERSION_REPLY_RECOGNIZED   1
-
-typedef struct nv_ioctl_query_device_intr
-{
-    NvU32 intrStatus NV_ALIGN_BYTES(4);
-    NvU32 status;
-} nv_ioctl_query_device_intr;
-
 #define NV_RM_DEVICE_INTR_ADDRESS 0x100
 
-
-/* system parameters that the kernel driver may use for configuration */
-typedef struct nv_ioctl_sys_params
-{
-    NvU64 memblock_size NV_ALIGN_BYTES(8);
-} nv_ioctl_sys_params_t;
-
-#define NV_IOCTL_NUMA_INFO_MAX_BLACKLIST_ADDRESSES 64
-typedef struct blacklist_addresses
-{
-    NvU64 addresses[NV_IOCTL_NUMA_INFO_MAX_BLACKLIST_ADDRESSES] NV_ALIGN_BYTES(8);
-    NvU32 numEntries;
-} nv_blacklist_addresses_t;
-
-
-/* per-device NUMA memory info as assigned by the system */
-typedef struct nv_ioctl_numa_info
-{
-    NvS32 nid;
-    NvS32 status;
-    NvU64 memblock_size NV_ALIGN_BYTES(8);
-    NvU64 numa_mem_addr NV_ALIGN_BYTES(8);
-    NvU64 numa_mem_size NV_ALIGN_BYTES(8);
-    nv_blacklist_addresses_t blacklist_addresses NV_ALIGN_BYTES(8);
-} nv_ioctl_numa_info_t;
-
-/* set the status of the device NUMA memory */
-typedef struct nv_ioctl_set_numa_status
-{
-    NvS32 status;
-} nv_ioctl_set_numa_status_t;
-
-#define NV_IOCTL_NUMA_STATUS_DISABLED               0
-#define NV_IOCTL_NUMA_STATUS_OFFLINE                1
-#define NV_IOCTL_NUMA_STATUS_ONLINE_IN_PROGRESS     2
-#define NV_IOCTL_NUMA_STATUS_ONLINE                 3
-#define NV_IOCTL_NUMA_STATUS_ONLINE_FAILED          4
-#define NV_IOCTL_NUMA_STATUS_OFFLINE_IN_PROGRESS    5
-#define NV_IOCTL_NUMA_STATUS_OFFLINE_FAILED         6
-
 /*!
  * @brief The order of the display clocks in the below defined enum
  * should be synced with below mapping array and macro.
@@ -428,6 +238,7 @@ typedef enum
     NV_SOC_IRQ_DISPLAY_TYPE,
     NV_SOC_IRQ_DPAUX_TYPE,
     NV_SOC_IRQ_GPIO_TYPE,
+    NV_SOC_IRQ_HDACODEC_TYPE,
     NV_SOC_IRQ_INVALID_TYPE
 } nv_soc_irq_type_t;
 
@@ -489,6 +300,7 @@ typedef struct nv_state_t
     nv_soc_irq_info_t soc_irq_info[NV_MAX_SOC_IRQS];
     NvS32 current_soc_irq;
     NvU32 num_soc_irqs;
+    NvU32 hdacodec_irq;
 
     NvBool primary_vga;
 
@@ -506,6 +318,7 @@ typedef struct nv_state_t
 
     /* Kernel interface DMA device data */
     nv_dma_device_t *dma_dev;
+    nv_dma_device_t *niso_dma_dev;
 
     /*
      * Per-GPU queue.  The actual queue object is usually allocated in the
@@ -538,8 +351,10 @@ typedef struct nv_state_t
 
 struct nv_file_private_t
 {
-    NvHandle hExportedRmObject[NV_MAX_RM_EXPORTED_OBJECTS];
-    NvU32 numExportedRmObjects;
+    NvHandle *handles;
+    NvU16 maxHandles;
+    NvU32 deviceInstance;
+    NvU8 metadata[64];
 };
 
 // Forward define the gpu ops structures
@@ -578,7 +393,7 @@ typedef NV_STATUS (*nvPmaEvictRangeCallback)(void *, NvU64, NvU64);
  */
 
 #define NV_FLAG_OPEN                   0x0001
-#define NV_FLAG_BLACKLIST              0x0002
+#define NV_FLAG_EXCLUDE                0x0002
 #define NV_FLAG_CONTROL                0x0004
 // Unused                              0x0008
 #define NV_FLAG_SOC_DISPLAY            0x0010
@@ -641,6 +456,12 @@ typedef enum
 #define NV_SYSTEM_ACPI_BATTERY_POWER_EVENT   0x8002
 #define NV_SYSTEM_ACPI_DOCK_EVENT            0x8003
 
+/*
+ * GPU add/remove events
+ */
+#define NV_SYSTEM_GPU_ADD_EVENT             0x9001
+#define NV_SYSTEM_GPU_REMOVE_EVENT          0x9002
+
 /*
  * Status bit definitions for display switch hotkey events.
  */
@@ -875,9 +696,25 @@ NV_STATUS NV_API_CALL nv_get_current_irq_priv_data(nv_state_t *, NvU32 *);
 NV_STATUS NV_API_CALL nv_acquire_fabric_mgmt_cap (int, int*);
 int       NV_API_CALL nv_cap_drv_init(void);
 void      NV_API_CALL nv_cap_drv_exit(void);
+NvBool    NV_API_CALL nv_is_gpu_accessible(nv_state_t *);
+
+struct dma_buf;
+typedef struct nv_dma_buf nv_dma_buf_t;
+
+NV_STATUS NV_API_CALL nv_dma_import_dma_buf      (nv_dma_device_t *, struct dma_buf *, NvU32 *, void **, nv_dma_buf_t **);
+NV_STATUS NV_API_CALL nv_dma_import_from_fd      (nv_dma_device_t *, NvS32, NvU32 *, void **, nv_dma_buf_t **);
+void      NV_API_CALL nv_dma_release_dma_buf     (void *, nv_dma_buf_t *);
+
+void      NV_API_CALL nv_schedule_uvm_isr        (nv_state_t *);
+
+
+NvBool    NV_API_CALL nv_platform_supports_s0ix  (void);
+NvBool    NV_API_CALL nv_s2idle_pm_configured    (void);
+
+
+
+
 
-NV_STATUS NV_API_CALL nv_dma_import_from_fd      (nv_dma_device_t *, NvS32, NvU32 *, void **, void **);
-void      NV_API_CALL nv_dma_release_from_fd     (void *, void *);
 
 
 
@@ -924,8 +761,7 @@ void       NV_API_CALL  rm_free_private_state    (nvidia_stack_t *, nv_state_t *
 NvBool     NV_API_CALL  rm_init_adapter          (nvidia_stack_t *, nv_state_t *);
 void       NV_API_CALL  rm_disable_adapter       (nvidia_stack_t *, nv_state_t *);
 void       NV_API_CALL  rm_shutdown_adapter      (nvidia_stack_t *, nv_state_t *);
-NV_STATUS  NV_API_CALL  rm_blacklist_adapter     (nvidia_stack_t *, nv_state_t *);
-NV_STATUS  NV_API_CALL  rm_validate_mmap_request (nvidia_stack_t *, nv_state_t *, nv_file_private_t *, NvU64, NvU64, NvU32 *, void **, NvU64 *);
+NV_STATUS  NV_API_CALL  rm_exclude_adapter       (nvidia_stack_t *, nv_state_t *);
 NV_STATUS  NV_API_CALL  rm_acquire_api_lock      (nvidia_stack_t *);
 NV_STATUS  NV_API_CALL  rm_release_api_lock      (nvidia_stack_t *);
 NV_STATUS  NV_API_CALL  rm_ioctl                 (nvidia_stack_t *, nv_state_t *, nv_file_private_t *, NvU32, void *, NvU32);
@@ -955,8 +791,14 @@ const char* NV_API_CALL rm_get_device_name       (NvU16, NvU16, NvU16);
 
 NvU64      NV_API_CALL  nv_rdtsc                 (void);
 
-NvBool     NV_API_CALL  rm_is_legacy_device      (NvU16, NvU16, NvU16, NvBool);
 NV_STATUS  NV_API_CALL  rm_is_supported_device   (nvidia_stack_t *, nv_state_t *);
+NvBool     NV_API_CALL  rm_is_supported_pci_device(NvU8   pci_class,
+                                                   NvU8   pci_subclass,
+                                                   NvU16  vendor,
+                                                   NvU16  device,
+                                                   NvU16  subsystem_vendor,
+                                                   NvU16  subsystem_device,
+                                                   NvBool print_legacy_warning);
 
 void       NV_API_CALL  rm_i2c_remove_adapters    (nvidia_stack_t *, nv_state_t *);
 NvBool     NV_API_CALL  rm_i2c_is_smbus_capable   (nvidia_stack_t *, nv_state_t *, void *);
@@ -973,7 +815,6 @@ NV_STATUS  NV_API_CALL  rm_p2p_get_pages          (nvidia_stack_t *, NvU64, NvU3
 NV_STATUS  NV_API_CALL  rm_p2p_register_callback  (nvidia_stack_t *, NvU64, NvU64, NvU64, void *, void (*)(void *), void *);
 NV_STATUS  NV_API_CALL  rm_p2p_put_pages          (nvidia_stack_t *, NvU64, NvU32, NvU64, void *);
 NV_STATUS  NV_API_CALL  rm_p2p_dma_map_pages      (nvidia_stack_t *, nv_dma_device_t *, NvU8 *, NvU32, NvU32, NvU64 *, void **);
-NV_STATUS  NV_API_CALL  rm_get_cpu_type           (nvidia_stack_t *, nv_cpu_type_t *);
 NV_STATUS  NV_API_CALL  rm_log_gpu_crash          (nv_stack_t *, nv_state_t *);
 
 NV_STATUS  NV_API_CALL  rm_gpu_ops_create_session (nvidia_stack_t *, nvgpuSessionHandle_t *);
@@ -1029,10 +870,8 @@ NV_STATUS  NV_API_CALL rm_gpu_ops_p2p_object_create(nvidia_stack_t *, nvgpuAddre
 void       NV_API_CALL rm_gpu_ops_p2p_object_destroy(nvidia_stack_t *, nvgpuSessionHandle_t, NvHandle);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_external_alloc_ptes(nvidia_stack_t*, nvgpuAddressSpaceHandle_t, NvHandle, NvU64, NvU64, nvgpuExternalMappingInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_retain_channel(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, void **, nvgpuChannelInstanceInfo_t);
-NV_STATUS  NV_API_CALL rm_gpu_ops_retain_channel_resources(nvidia_stack_t *, void *, nvgpuChannelResourceInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_bind_channel_resources(nvidia_stack_t *, void *, nvgpuChannelResourceBindParams_t);
 void       NV_API_CALL rm_gpu_ops_release_channel(nvidia_stack_t *, void *);
-void       NV_API_CALL rm_gpu_ops_release_channel_resources(nvidia_stack_t *, NvP64*, NvU32);
 void       NV_API_CALL rm_gpu_ops_stop_channel(nvidia_stack_t *, void *, NvBool);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_channel_resource_ptes(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvP64, NvU64, NvU64, nvgpuExternalMappingInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_report_non_replayable_fault(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, const void *);
@@ -1044,7 +883,7 @@ NV_STATUS  NV_API_CALL rm_gpu_need_4k_page_isolation(nv_state_t *, NvBool *);
 NvBool     NV_API_CALL rm_is_chipset_io_coherent(nv_stack_t *);
 NvBool     NV_API_CALL rm_init_event_locks(nvidia_stack_t *, nv_state_t *);
 void       NV_API_CALL rm_destroy_event_locks(nvidia_stack_t *, nv_state_t *);
-NV_STATUS  NV_API_CALL rm_get_gpu_numa_info(nvidia_stack_t *, nv_state_t *, NvS32 *, NvU64 *, NvU64 *, nv_blacklist_addresses_t *);
+NV_STATUS  NV_API_CALL rm_get_gpu_numa_info(nvidia_stack_t *, nv_state_t *, NvS32 *, NvU64 *, NvU64 *, NvU64 *, NvU32 *);
 NV_STATUS  NV_API_CALL rm_set_backlight(nvidia_stack_t *, nv_state_t *, NvU32, NvU32);
 NV_STATUS  NV_API_CALL rm_get_backlight(nvidia_stack_t *, nv_state_t *, NvU32, NvU32 *);
 NV_STATUS  NV_API_CALL rm_gpu_numa_online(nvidia_stack_t *, nv_state_t *);
@@ -1054,6 +893,7 @@ void       NV_API_CALL rm_check_for_gpu_surprise_removal(nvidia_stack_t *, nv_st
 NV_STATUS  NV_API_CALL rm_set_external_kernel_client_count(nvidia_stack_t *, nv_state_t *, NvBool);
 NV_STATUS  NV_API_CALL rm_schedule_gpu_wakeup(nvidia_stack_t *, nv_state_t *);
 NvBool     NV_API_CALL rm_is_iommu_needed_for_sriov(nvidia_stack_t *, nv_state_t *);
+NvBool     NV_API_CALL rm_disable_iomap_wc(void);
 
 void       NV_API_CALL rm_init_dynamic_power_management(nvidia_stack_t *, nv_state_t *, NvBool);
 void       NV_API_CALL rm_cleanup_dynamic_power_management(nvidia_stack_t *, nv_state_t *);
@@ -1074,13 +914,15 @@ NV_STATUS  NV_API_CALL  nv_vgpu_get_type_info(nvidia_stack_t *, nv_state_t *, Nv
 NV_STATUS  NV_API_CALL  nv_vgpu_get_bar_info(nvidia_stack_t *, nv_state_t *, const NvU8 *, NvU64 *, NvU32, void *, void **);
 NV_STATUS  NV_API_CALL  nv_vgpu_start(nvidia_stack_t *, const NvU8 *, void *, NvS32 *, NvU8 *, NvU32);
 NV_STATUS  NV_API_CALL  nv_vgpu_get_sparse_mmap(nvidia_stack_t *, nv_state_t *, const NvU8 *, NvU64 **, NvU64 **, NvU32 *);
-NV_STATUS  NV_API_CALL  nv_vgpu_process_vf_info(nvidia_stack_t *, nv_state_t *, NvU8, NvU32, NvU8, NvU8, NvU8);
+NV_STATUS  NV_API_CALL  nv_vgpu_process_vf_info(nvidia_stack_t *, nv_state_t *, NvU8, NvU32, NvU8, NvU8, NvU8, NvBool, void *);
 NV_STATUS  NV_API_CALL  nv_vgpu_update_request(nvidia_stack_t *, const NvU8 *, NvU32, NvU64 *, NvU64 *, const char *);
 NV_STATUS  NV_API_CALL  nv_gpu_bind_event(nvidia_stack_t *);
 
 NV_STATUS NV_API_CALL nv_get_usermap_access_params(nv_state_t*, nv_usermap_access_params_t*);
 nv_soc_irq_type_t NV_API_CALL nv_get_current_irq_type(nv_state_t*);
 
+/* Callbacks should occur roughly every 10ms. */
+#define NV_SNAPSHOT_TIMER_HZ 100
 void NV_API_CALL nv_start_snapshot_timer(void (*snapshot_callback)(void *context));
 void NV_API_CALL nv_flush_snapshot_timer(void);
 void NV_API_CALL nv_stop_snapshot_timer(void);
