@@ -16,9 +16,9 @@
 #define _NV_UVM_TYPES_H_
 
 #include "nvtypes.h"
+#include "nvstatus.h"
 #include "nvgputypes.h"
 #include "nvCpuUuid.h"
-#include "uvmtypes.h"
 #ifndef __KERNEL__
 
 #endif
@@ -33,10 +33,8 @@
 #define UVM_PAGE_SIZE_64K        0x10000
 #define UVM_PAGE_SIZE_128K       0x20000
 #define UVM_PAGE_SIZE_2M         0x200000
-
 #define UVM_PAGE_SIZE_512M       0x20000000
 
-
 //
 // When modifying flags, make sure they are compatible with the mirrored
 // PMA_* flags in pma.h.
@@ -132,6 +130,9 @@ typedef struct UvmGpuChannelResourceInfo_tag
     // Out: Ptr to the RM memDesc of the channel resource.
     NvP64 resourceDescriptor;
 
+    // Out: RM ID of the channel resource.
+    NvU32 resourceId;
+
     // Out: Set to NV_TRUE, if the resource is global.
     //      Set to NV_FALSE, if the resource is local.
     NvBool globalResource;
@@ -149,6 +150,8 @@ typedef enum
     UVM_GPU_CHANNEL_ENGINE_TYPE_CE = 2,
 } UVM_GPU_CHANNEL_ENGINE_TYPE;
 
+#define UVM_GPU_CHANNEL_MAX_RESOURCES   13
+
 typedef struct UvmGpuChannelInstanceInfo_tag
 {
     // Out: Starting address of the channel instance.
@@ -176,8 +179,11 @@ typedef struct UvmGpuChannelInstanceInfo_tag
     // Out: Maximum number of subcontexts in the TSG to which the channel belongs
     NvU32 tsgMaxSubctxCount;
 
-    // Out: Number of channel resources (local CTX buffers + global CTX buffers)
+    // Out: Info of channel resources (local CTX buffers + global CTX buffers)
     // associated with the channel.
+    UvmGpuChannelResourceInfo resourceInfo[UVM_GPU_CHANNEL_MAX_RESOURCES];
+
+    // Out: Number of valid entries in resourceInfo array.
     NvU32 resourceCount;
 
     // Out: Type of the engine the channel is bound to
@@ -192,7 +198,6 @@ typedef struct UvmGpuChannelInstanceInfo_tag
     // Out: Channel handle to be used in the CLEAR_FAULTED method
     NvU32 clearFaultedToken;
 
-
     // Out: Address of the NV_CHRAM_CHANNEL register required to clear the
     // ENG_FAULTED/PBDMA_FAULTED bits after servicing non-replayable faults on
     // Ampere+ GPUs
@@ -206,7 +211,6 @@ typedef struct UvmGpuChannelInstanceInfo_tag
     // is bound to, or zero if the GPU does not support SMC or it is a CE
     // channel
     NvU32 smcEngineVeIdOffset;
-
 } UvmGpuChannelInstanceInfo;
 
 typedef struct UvmGpuChannelResourceBindParams_tag
@@ -214,6 +218,9 @@ typedef struct UvmGpuChannelResourceBindParams_tag
     // In: Ptr to the RM memDesc of the channel resource.
     NvP64 resourceDescriptor;
 
+    // In: RM ID of the channel resource.
+    NvU32 resourceId;
+
     // In: Starting VA at which the channel resource is mapped.
     NvU64 resourceVa;
 
@@ -241,7 +248,18 @@ typedef struct UvmGpuChannelInfo_tag
     // submission token requires the channel to be bound to a runlist and that
     // happens after CE allocation.
     volatile NvU32    *workSubmissionOffset;
+
+    // To be deprecated. See pWorkSubmissionToken below.
     NvU32              workSubmissionToken;
+
+    //
+    // This is the memory location where the most recently updated work
+    // submission token for this channel will be written to. After submitting
+    // new work and updating GP_PUT with the appropriate fence, the token must
+    // be read from this location before writing it to the workSubmissionOffset
+    // to kick off the new work.
+    //
+    volatile NvU32    *pWorkSubmissionToken;
 } UvmGpuChannelInfo;
 
 typedef enum
@@ -320,9 +338,7 @@ typedef enum
     UVM_PEER_LINK_TYPE_PCIE,
     UVM_PEER_LINK_TYPE_NVLINK_1,
     UVM_PEER_LINK_TYPE_NVLINK_2,
-
     UVM_PEER_LINK_TYPE_NVLINK_3,
-
 } UVM_PEER_LINK_TYPE;
 
 typedef struct UvmGpuCaps_tag
@@ -371,13 +387,11 @@ typedef struct UvmGpuAddressSpaceInfo_tag
     // Maximum number of subcontexts supported under this GPU address space
     NvU32           maxSubctxCount;
 
-
     NvBool          smcEnabled;
 
     NvU32           smcSwizzId;
 
     NvU32           smcGpcCount;
-
 } UvmGpuAddressSpaceInfo;
 
 typedef struct UvmGpuAllocInfo_tag
@@ -393,22 +407,71 @@ typedef struct UvmGpuAllocInfo_tag
     NvHandle hPhysHandle;           // Handle for phys allocation either provided or retrieved
 } UvmGpuAllocInfo;
 
+// !!! The following enums (with UvmRm prefix) are defined and documented in
+// mm/uvm/interface/uvm_types.h and must be mirrored. Please refer to that file
+// for more details.
+
+// UVM GPU mapping types
+typedef enum
+{
+    UvmRmGpuMappingTypeDefault = 0,
+    UvmRmGpuMappingTypeReadWriteAtomic = 1,
+    UvmRmGpuMappingTypeReadWrite = 2,
+    UvmRmGpuMappingTypeReadOnly = 3,
+    UvmRmGpuMappingTypeCount = 4
+} UvmRmGpuMappingType;
+
+// UVM GPU caching types
+typedef enum
+{
+    UvmRmGpuCachingTypeDefault = 0,
+    UvmRmGpuCachingTypeForceUncached = 1,
+    UvmRmGpuCachingTypeForceCached = 2,
+    UvmRmGpuCachingTypeCount = 3 
+} UvmRmGpuCachingType;
+
+// UVM GPU format types
+typedef enum {
+   UvmRmGpuFormatTypeDefault = 0,
+   UvmRmGpuFormatTypeBlockLinear = 1,
+   UvmRmGpuFormatTypeCount = 2 
+} UvmRmGpuFormatType;
+
+// UVM GPU Element bits types
+typedef enum {
+   UvmRmGpuFormatElementBitsDefault = 0,
+   UvmRmGpuFormatElementBits8 = 1,
+   UvmRmGpuFormatElementBits16 = 2,
+   // Cuda does not support 24-bit width
+   UvmRmGpuFormatElementBits32 = 4,
+   UvmRmGpuFormatElementBits64 = 5,
+   UvmRmGpuFormatElementBits128 = 6,
+   UvmRmGpuFormatElementBitsCount = 7
+} UvmRmGpuFormatElementBits;
+
+// UVM GPU Compression types
+typedef enum {
+    UvmRmGpuCompressionTypeDefault = 0,
+    UvmRmGpuCompressionTypeEnabledNoPlc = 1,
+    UvmRmGpuCompressionTypeCount = 2
+} UvmRmGpuCompressionType;
+
 typedef struct UvmGpuExternalMappingInfo_tag
 {
     // In: GPU caching ability.
-    UvmGpuCachingType cachingType;
+    UvmRmGpuCachingType cachingType;
 
     // In: Virtual permissions.
-    UvmGpuMappingType mappingType;
+    UvmRmGpuMappingType mappingType;
 
     // In: RM virtual mapping memory format
-    UvmGpuFormatType formatType;
+    UvmRmGpuFormatType formatType;
 
     // In: RM virtual mapping element bits
-    UvmGpuFormatElementBits elementBits;
+    UvmRmGpuFormatElementBits elementBits;
 
     // In: RM virtual compression type
-    UvmGpuCompressionType compressionType;
+    UvmRmGpuCompressionType compressionType;
 
     // In: Size of the buffer to store PTEs (in bytes).
     NvU64 pteBufferSize;
@@ -431,27 +494,30 @@ typedef struct UvmGpuExternalMappingInfo_tag
 
 typedef struct UvmGpuP2PCapsParams_tag
 {
-    NvU32 peerIds[2];               // peerId[i] contains gpu[i]'s peer id of gpu[1 - i]
-    NvU32 p2pLink;                  // UVM_PEER_LINK_TYPE
-    NvU32 optimalNvlinkWriteCEs[2]; // optimalNvlinkWriteCEs[i] contains gpu[i]'s optimal
-                                    // CE for writing to gpu[1 - i] over NVLink.
-                                    // The CE indexes are valid only if the GPUs are direct
-                                    // NVLink peers
-                                    //
-                                    // The value returned by RM for this field
-                                    // may change when a GPU is registered with
-                                    // RM for the first time, so UVM needs to
-                                    // query it again each time a GPU is
-                                    // registered.
-    NvU32 totalLinkLineRateMBps;    // Maximum unidirectional bandwidth between
-                                    // the peers in megabytes per second, not
-                                    // taking into account the protocols
-                                    // overhead. The reported bandwidth for
-                                    // indirect peers is zero.
-    NvU32 indirectAccess      : 1;  // Whether peers have a indirect link to communicate.
-                                    // On P9 systems, this is true if peers are
-                                    // connected to different NPUs that forward the
-                                    // requests between them.
+    // Out: peerId[i] contains gpu[i]'s peer id of gpu[1 - i]. Only defined if
+    // the GPUs are direct peers.
+    NvU32 peerIds[2];
+
+    // Out: UVM_PEER_LINK_TYPE
+    NvU32 p2pLink;
+
+    // Out: optimalNvlinkWriteCEs[i] contains gpu[i]'s optimal CE for writing to
+    // gpu[1 - i]. The CE indexes are valid only if the GPUs are NVLink peers.
+    //
+    // The value returned by RM for this field may change when a GPU is
+    // registered with RM for the first time, so UVM needs to query it again
+    // each time a GPU is registered.
+    NvU32 optimalNvlinkWriteCEs[2];
+
+    // Out: Maximum unidirectional bandwidth between the peers in megabytes per
+    // second, not taking into account the protocols overhead. The reported
+    // bandwidth for indirect peers is zero.
+    NvU32 totalLinkLineRateMBps;
+
+    // Out: True if the peers have a indirect link to communicate. On P9
+    // systems, this is true if peers are connected to different NPUs that
+    // forward the requests between them.
+    NvU32 indirectAccess      : 1;
 } UvmGpuP2PCapsParams;
 
 // Platform-wide information
@@ -465,9 +531,7 @@ typedef struct UvmGpuClientInfo_tag
 {
     NvHandle hClient;
 
-
     NvHandle hSmcPartRef;
-
 } UvmGpuClientInfo;
 
 #define UVM_GPU_NAME_LENGTH 0x40
@@ -514,17 +578,13 @@ typedef struct UvmGpuInfo_tag
     NvBool isSimulated;
 
     // Number of GPCs
-
     // If SMC is enabled, this is the currently configured number of GPCs for
     // the given partition (also see the smcSwizzId field below).
-
     NvU32 gpcCount;
 
     // Maximum number of GPCs; NV_SCAL_LITTER_NUM_GPCS
-
     // This number is independent of the partition configuration, and can be
     // used to conservatively size GPU-global constructs.
-
     NvU32 maxGpcCount;
 
     // Number of TPCs
@@ -533,7 +593,6 @@ typedef struct UvmGpuInfo_tag
     // Maximum number of TPCs per GPC
     NvU32 maxTpcPerGpcCount;
 
-
     // NV_TRUE if SMC is enabled on this GPU.
     NvBool smcEnabled;
 
@@ -544,7 +603,6 @@ typedef struct UvmGpuInfo_tag
     NvU32 smcSwizzId;
 
     UvmGpuClientInfo smcUserClientInfo;
-
 } UvmGpuInfo;
 
 typedef struct UvmGpuVaAllocInfo_tag
@@ -725,6 +783,11 @@ typedef struct UvmGpuFaultInfo_tag
         // Shadow buffer for non-replayable faults on cpu memory. Resman copies
         // here the non-replayable faults that need to be handled by UVM
         void* shadowBufferAddress;
+
+        // Eecution context for the queue associated with the fault buffer
+        void* shadowBufferContext;
+
+        // Fault buffer size
         NvU32  bufferSize;
 
         // Preallocated stack for functions called from the UVM isr top half
