@@ -34,16 +34,14 @@
 #include "nvidia-drm-ioctl.h"
 #include "nvidia-drm-format.h"
 
+#include "nvmisc.h"
+
 #include <drm/drm_crtc_helper.h>
 #include <drm/drm_plane_helper.h>
 
 #include <drm/drm_atomic.h>
 #include <drm/drm_atomic_helper.h>
 
-#if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
-#include <drm/drm_blend.h>
-#endif
-
 static void nv_drm_plane_destroy(struct drm_plane *plane)
 {
     /* plane->state gets freed here */
@@ -88,6 +86,45 @@ plane_req_config_update(struct drm_plane *plane,
         },
     };
 
+#if defined(NV_DRM_ROTATION_AVAILABLE)
+    /*
+     * plane_state->rotation is only valid when plane->rotation_property
+     * is non-NULL.
+     */
+    if (plane->rotation_property != NULL) {
+        if (plane_state->rotation & DRM_MODE_REFLECT_X) {
+            req_config->config.rrParams.reflectionX = true;
+        }
+
+        if (plane_state->rotation & DRM_MODE_REFLECT_Y) {
+            req_config->config.rrParams.reflectionY = true;
+        }
+
+        switch (plane_state->rotation & DRM_MODE_ROTATE_MASK) {
+            case DRM_MODE_ROTATE_0:
+                req_config->config.rrParams.rotation = NVKMS_ROTATION_0;
+                break;
+            case DRM_MODE_ROTATE_90:
+                req_config->config.rrParams.rotation = NVKMS_ROTATION_90;
+                break;
+            case DRM_MODE_ROTATE_180:
+                req_config->config.rrParams.rotation = NVKMS_ROTATION_180;
+                break;
+            case DRM_MODE_ROTATE_270:
+                req_config->config.rrParams.rotation = NVKMS_ROTATION_270;
+                break;
+            default:
+                /*
+                 * We should not hit this, because
+                 * plane_state->rotation should only have values
+                 * registered in
+                 * __nv_drm_plane_create_rotation_property().
+                 */
+                WARN_ON("Unsupported rotation");
+                break;
+        }
+    }
+#endif
 
 #if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
     if (plane->blend_mode_property != NULL && plane->alpha_property != NULL) {
@@ -106,7 +143,7 @@ plane_req_config_update(struct drm_plane *plane,
                  * We should not hit this, because
                  * plane_state->pixel_blend_mode should only have values
                  * registered in
-                 * __nv_drm_cursor_create_alpha_blending_properties().
+                 * __nv_drm_plane_create_alpha_blending_properties().
                  */
                 WARN_ON("Unsupported blending mode");
                 break;
@@ -132,7 +169,7 @@ plane_req_config_update(struct drm_plane *plane,
                  * We should not hit this, because
                  * plane_state->pixel_blend_mode should only have values
                  * registered in
-                 * __nv_drm_cursor_create_alpha_blending_properties().
+                 * __nv_drm_plane_create_alpha_blending_properties().
                  */
                 WARN_ON("Unsupported blending mode");
                 break;
@@ -419,7 +456,7 @@ static int head_modeset_config_attach_connector(
 
 /**
  * nv_drm_crtc_atomic_check() can fail after it has modified
- * the 'nv_drm_crtc_state::req_config', that is fine becase 'nv_drm_crtc_state'
+ * the 'nv_drm_crtc_state::req_config', that is fine because 'nv_drm_crtc_state'
  * will be discarded if ->atomic_check() fails.
  */
 static int nv_drm_crtc_atomic_check(struct drm_crtc *crtc,
@@ -519,10 +556,95 @@ static const struct drm_crtc_helper_funcs nv_crtc_helper_funcs = {
     .mode_fixup = nv_drm_crtc_mode_fixup,
 };
 
+static void
+__nv_drm_plane_create_alpha_blending_properties(struct drm_plane *plane,
+                                                 NvU32 validCompModes)
+{
+#if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
+    if ((validCompModes &
+         NVBIT(NVKMS_COMPOSITION_MODE_PREMULT_SURFACE_ALPHA)) != 0x0 &&
+        (validCompModes &
+         NVBIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_SURFACE_ALPHA)) != 0x0) {
+
+        drm_plane_create_alpha_property(plane);
+        drm_plane_create_blend_mode_property(plane,
+                                             NVBIT(DRM_MODE_BLEND_PREMULTI) |
+                                             NVBIT(DRM_MODE_BLEND_COVERAGE));
+    } else if ((validCompModes &
+                NVBIT(NVKMS_COMPOSITION_MODE_PREMULT_ALPHA)) != 0x0 &&
+               (validCompModes &
+                NVBIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_ALPHA)) != 0x0) {
+
+        drm_plane_create_blend_mode_property(plane,
+                                             NVBIT(DRM_MODE_BLEND_PREMULTI) |
+                                             NVBIT(DRM_MODE_BLEND_COVERAGE));
+   }
+#endif
+}
+
+static void
+__nv_drm_plane_create_rotation_property(struct drm_plane *plane,
+                                        NvU16 validLayerRRTransforms)
+{
+#if defined(NV_DRM_ROTATION_AVAILABLE)
+    enum NvKmsRotation curRotation;
+    NvU32 supported_rotations = 0;
+    struct NvKmsRRParams rrParams = {
+        .rotation = NVKMS_ROTATION_0,
+        .reflectionX = true,
+        .reflectionY = true,
+    };
+
+    if ((NVBIT(NvKmsRRParamsToCapBit(&rrParams)) &
+        validLayerRRTransforms) != 0) {
+        supported_rotations |= DRM_MODE_REFLECT_X;
+        supported_rotations |= DRM_MODE_REFLECT_Y;
+    }
+
+    rrParams.reflectionX = false;
+    rrParams.reflectionY = false;
+
+    for (curRotation = NVKMS_ROTATION_MIN;
+         curRotation <= NVKMS_ROTATION_MAX; curRotation++) {
+        rrParams.rotation = curRotation;
+        if ((NVBIT(NvKmsRRParamsToCapBit(&rrParams)) &
+                    validLayerRRTransforms) == 0) {
+            continue;
+        }
+
+        switch (curRotation) {
+            case NVKMS_ROTATION_0:
+                supported_rotations |= DRM_MODE_ROTATE_0;
+                break;
+            case NVKMS_ROTATION_90:
+                supported_rotations |= DRM_MODE_ROTATE_90;
+                break;
+            case NVKMS_ROTATION_180:
+                supported_rotations |= DRM_MODE_ROTATE_180;
+                break;
+            case NVKMS_ROTATION_270:
+                supported_rotations |= DRM_MODE_ROTATE_270;
+                break;
+            default:
+                break;
+        }
+
+    }
+
+    if (supported_rotations != 0) {
+        drm_plane_create_rotation_property(plane, DRM_MODE_ROTATE_0,
+                                           supported_rotations);
+    }
+#endif
+}
+
 static struct drm_plane*
 nv_drm_plane_create(struct drm_device *dev,
                     enum drm_plane_type plane_type,
-                    const u32 formats[], unsigned int formats_count)
+                    NvU32 head,
+                    long unsigned int nvkms_formats_mask,
+                    NvU32 validCompositionModes,
+                    NvU16 validLayerRRTransforms)
 {
 #if defined(NV_DRM_UNIVERSAL_PLANE_INIT_HAS_FORMAT_MODIFIERS_ARG)
     struct nv_drm_device *nv_dev = to_nv_device(dev);
@@ -533,9 +655,18 @@ nv_drm_plane_create(struct drm_device *dev,
 #endif
     struct drm_plane *plane = NULL;
     int ret = -ENOMEM;
+    uint32_t *formats = NULL;
+    unsigned int formats_count = 0;
+
+    formats =
+        nv_drm_format_array_alloc(&formats_count,
+                                  nvkms_formats_mask);
+    if (formats == NULL) {
+        goto failed;
+    }
 
     if ((plane = nv_drm_calloc(1, sizeof(*plane))) == NULL) {
-        goto failed;
+        goto failed_plane_alloc;
     }
 
     plane->state = nv_drm_calloc(1, sizeof(*plane->state));
@@ -546,13 +677,16 @@ nv_drm_plane_create(struct drm_device *dev,
     plane->state->plane = plane;
 
     /*
-     * Possible_crtcs is zero here because drm_crtc_init_with_planes() will
-     * assign the plane's possible_crtcs after the crtc is successfully
-     * initialized.
+     * Possible_crtcs for primary and cursor plane is zero because
+     * drm_crtc_init_with_planes() will assign the plane's possible_crtcs
+     * after the crtc is successfully initialized.
      */
     ret = drm_universal_plane_init(
         dev,
-        plane, 0 /* possible_crtcs */, &nv_plane_funcs,
+        plane,
+        (plane_type == DRM_PLANE_TYPE_OVERLAY) ?
+        (1 << head) : 0,
+        &nv_plane_funcs,
         formats, formats_count,
 #if defined(NV_DRM_UNIVERSAL_PLANE_INIT_HAS_FORMAT_MODIFIERS_ARG)
         (plane_type == DRM_PLANE_TYPE_CURSOR) ?
@@ -570,6 +704,14 @@ nv_drm_plane_create(struct drm_device *dev,
 
     drm_plane_helper_add(plane, &nv_plane_helper_funcs);
 
+    __nv_drm_plane_create_alpha_blending_properties(
+            plane,
+            validCompositionModes);
+
+    __nv_drm_plane_create_rotation_property(
+            plane,
+            validLayerRRTransforms);
+
     return plane;
 
 failed_plane_init:
@@ -578,6 +720,9 @@ failed_plane_init:
 failed_state_alloc:
     nv_drm_free(plane);
 
+failed_plane_alloc:
+    nv_drm_free(formats);
+
 failed:
     return ERR_PTR(ret);
 }
@@ -642,60 +787,34 @@ failed:
     return ERR_PTR(ret);
 }
 
-static void
-__nv_drm_cursor_create_alpha_blending_properties(struct drm_plane *cursor_plane,
-                                                 NvU32 validCompModes)
-{
-#if defined(NV_DRM_ALPHA_BLENDING_AVAILABLE)
-    if ((validCompModes &
-         BIT(NVKMS_COMPOSITION_MODE_PREMULT_SURFACE_ALPHA)) != 0x0 &&
-        (validCompModes &
-         BIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_SURFACE_ALPHA)) != 0x0) {
-
-        drm_plane_create_alpha_property(cursor_plane);
-        drm_plane_create_blend_mode_property(cursor_plane,
-                                             BIT(DRM_MODE_BLEND_PREMULTI) |
-                                             BIT(DRM_MODE_BLEND_COVERAGE));
-    } else if ((validCompModes &
-                BIT(NVKMS_COMPOSITION_MODE_PREMULT_ALPHA)) != 0x0 &&
-               (validCompModes &
-                BIT(NVKMS_COMPOSITION_MODE_NON_PREMULT_ALPHA)) != 0x0) {
-
-        drm_plane_create_blend_mode_property(cursor_plane,
-                                             BIT(DRM_MODE_BLEND_PREMULTI) |
-                                             BIT(DRM_MODE_BLEND_COVERAGE));
-   }
-#endif
-}
-
 void nv_drm_enumerate_crtcs_and_planes(
     struct nv_drm_device *nv_dev,
-    const long unsigned int nvkms_primary_formats_mask,
-    const long unsigned int nvkms_cursor_formats_mask,
-    unsigned int num_heads,
-    NvU32 validCursorCompositionModes)
+    const struct NvKmsKapiDeviceResourcesInfo *pResInfo)
 {
     unsigned int i;
-    unsigned int cursor_formats_count = 0;
-    unsigned int primary_formats_count = 0;
-    uint32_t *primary_formats =
-        nv_drm_format_array_alloc(&primary_formats_count,
-                                  nvkms_primary_formats_mask);
-    uint32_t *cursor_formats =
-        nv_drm_format_array_alloc(&cursor_formats_count,
-                                  nvkms_cursor_formats_mask);
 
-    if (primary_formats == NULL || cursor_formats == NULL) {
-        goto done;
-    }
+    NvU32 validCursorCompositionModes =
+        pResInfo->caps.validCursorCompositionModes;
+    NvU32 validOverlayCompositionModes =
+        pResInfo->caps.validOverlayCompositionModes;
+    long unsigned int nvkms_primary_formats_mask =
+        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_PRIMARY];
+    long unsigned int nvkms_cursor_formats_mask =
+        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_CURSOR];
+    long unsigned int nvkms_overlay_formats_mask =
+        pResInfo->supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_OVERLAY];
+    NvU16 validLayerRRTransforms = pResInfo->caps.validLayerRRTransforms;
 
-    for (i = 0; i < num_heads; i++) {
+    for (i = 0; i < pResInfo->numHeads; i++) {
         struct drm_plane *primary_plane = NULL, *cursor_plane = NULL;
+        struct drm_plane *overlay_plane = NULL;
 
         primary_plane = nv_drm_plane_create(nv_dev->dev,
                                             DRM_PLANE_TYPE_PRIMARY,
-                                            primary_formats,
-                                            primary_formats_count);
+                                            i,
+                                            nvkms_primary_formats_mask,
+                                            0,
+                                            validLayerRRTransforms);
         if (IS_ERR(primary_plane)) {
             NV_DRM_DEV_LOG_ERR(
                 nv_dev,
@@ -703,19 +822,18 @@ void nv_drm_enumerate_crtcs_and_planes(
                 i, PTR_ERR(primary_plane));
         }
 
+        /* nvkms doesn't support rrTransforms for cursor layer */
         cursor_plane = nv_drm_plane_create(nv_dev->dev,
                                            DRM_PLANE_TYPE_CURSOR,
-                                           cursor_formats,
-                                           cursor_formats_count);
+                                           i,
+                                           nvkms_cursor_formats_mask,
+                                           validCursorCompositionModes,
+                                           0 /* rrTransforms */);
         if (IS_ERR(cursor_plane)) {
             NV_DRM_DEV_LOG_ERR(
                 nv_dev,
                 "Failed to create cursor plane for head %u, error = %ld",
                 i, PTR_ERR(cursor_plane));
-        } else {
-            __nv_drm_cursor_create_alpha_blending_properties(
-                cursor_plane,
-                validCursorCompositionModes);
         }
 
         if (primary_plane != NULL) {
@@ -725,17 +843,70 @@ void nv_drm_enumerate_crtcs_and_planes(
                                      i);
 
             if (IS_ERR(crtc)) {
+                nv_drm_plane_destroy(primary_plane);
+                nv_drm_plane_destroy(cursor_plane);
                 NV_DRM_DEV_LOG_ERR(
                     nv_dev,
                     "Failed to add DRM CRTC for head %u, error = %ld",
                     i, PTR_ERR(crtc));
+                continue;
             }
         }
+
+        overlay_plane = nv_drm_plane_create(nv_dev->dev,
+                                            DRM_PLANE_TYPE_OVERLAY,
+                                            i,
+                                            nvkms_overlay_formats_mask,
+                                            validOverlayCompositionModes,
+                                            validLayerRRTransforms);
+        if (IS_ERR(overlay_plane)) {
+            NV_DRM_DEV_LOG_ERR(
+                nv_dev,
+                "Failed to create overlay plane for head %u, error = %ld",
+                i, PTR_ERR(overlay_plane));
+        }
+    }
+}
+/*
+ * Helper function to convert NvKmsKapiCrcs to drm_nvidia_crtc_crc32_out.
+ */
+static void NvKmsKapiCrcsToDrm(const struct NvKmsKapiCrcs *crcs,
+                               struct drm_nvidia_crtc_crc32_v2_out *drmCrcs)
+{
+    drmCrcs->outputCrc32.value = crcs->outputCrc32.value;
+    drmCrcs->outputCrc32.supported = crcs->outputCrc32.supported;
+    drmCrcs->rasterGeneratorCrc32.value = crcs->rasterGeneratorCrc32.value;
+    drmCrcs->rasterGeneratorCrc32.supported = crcs->rasterGeneratorCrc32.supported;
+    drmCrcs->compositorCrc32.value = crcs->compositorCrc32.value;
+    drmCrcs->compositorCrc32.supported = crcs->compositorCrc32.supported;
+}
+
+int nv_drm_get_crtc_crc32_v2_ioctl(struct drm_device *dev,
+                                   void *data, struct drm_file *filep)
+{
+    struct drm_nvidia_get_crtc_crc32_v2_params *params = data;
+    struct nv_drm_device *nv_dev = to_nv_device(dev);
+    struct drm_crtc *crtc = NULL;
+    struct nv_drm_crtc *nv_crtc = NULL;
+    struct NvKmsKapiCrcs crc32;
+
+    if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
+        return -ENOENT;
     }
 
-done:
-    nv_drm_free(primary_formats);
-    nv_drm_free(cursor_formats);
+    crtc = nv_drm_crtc_find(dev, params->crtc_id);
+    if (!crtc) {
+        return -ENOENT;
+    }
+
+    nv_crtc = to_nv_crtc(crtc);
+
+    if (!nvKms->getCRC32(nv_dev->pDevice, nv_crtc->head, &crc32)) {
+        return -ENODEV;
+    }
+    NvKmsKapiCrcsToDrm(&crc32, &params->crc32);
+
+    return 0;
 }
 
 int nv_drm_get_crtc_crc32_ioctl(struct drm_device *dev,
@@ -745,30 +916,25 @@ int nv_drm_get_crtc_crc32_ioctl(struct drm_device *dev,
     struct nv_drm_device *nv_dev = to_nv_device(dev);
     struct drm_crtc *crtc = NULL;
     struct nv_drm_crtc *nv_crtc = NULL;
-    NvU32 crc32 = 0;
-    int ret = 0;
+    struct NvKmsKapiCrcs crc32;
 
     if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        ret = -ENOENT;
-        goto done;
+        return -ENOENT;
     }
 
     crtc = nv_drm_crtc_find(dev, params->crtc_id);
     if (!crtc) {
-        ret = -ENOENT;
-        goto done;
+        return -ENOENT;
     }
 
     nv_crtc = to_nv_crtc(crtc);
 
     if (!nvKms->getCRC32(nv_dev->pDevice, nv_crtc->head, &crc32)) {
-        ret = -ENODEV;
+        return -ENODEV;
     }
+    params->crc32 = crc32.outputCrc32.value;
 
-    params->crc32 = crc32;
-
-done:
-    return ret;
+    return 0;
 }
 
 #endif
