@@ -34,6 +34,7 @@
 #include "nvidia-drm-helper.h"
 #include "nvidia-drm-gem-nvkms-memory.h"
 #include "nvidia-drm-gem-user-memory.h"
+#include "nvidia-drm-gem-dma-buf.h"
 
 #if defined(NV_DRM_AVAILABLE)
 
@@ -414,12 +415,7 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
 
     /* Add crtcs */
 
-    nv_drm_enumerate_crtcs_and_planes(
-        nv_dev,
-        resInfo.supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_PRIMARY],
-        resInfo.supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_CURSOR],
-        resInfo.numHeads,
-        resInfo.caps.validCursorCompositionModes);
+    nv_drm_enumerate_crtcs_and_planes(nv_dev, &resInfo);
 
     /* Add connectors and encoders */
 
@@ -674,29 +670,6 @@ static struct drm_bus nv_drm_bus = {
 
 #endif /* NV_DRM_BUS_PRESENT */
 
-#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
-static int nv_drm_mmap(struct file *file, struct vm_area_struct *vma)
-{
-    struct drm_gem_object *gem;
-    struct nv_drm_gem_object *nv_gem;
-    int err;
-
-    err = drm_gem_mmap(file, vma);
-    if (err < 0)
-        return err;
-
-    gem = vma->vm_private_data;
-    nv_gem = to_nv_gem_object(gem);
-
-    if (!nv_gem || !nv_gem->ops->adjust_mmap_flags(nv_gem, vma)) {
-        drm_gem_vm_close(vma);
-        return -EINVAL;
-    }
-
-    return 0;
-}
-#endif
-
 static const struct file_operations nv_drm_fops = {
     .owner          = THIS_MODULE,
 
@@ -753,6 +726,9 @@ static const struct drm_ioctl_desc nv_drm_ioctls[] = {
     DRM_IOCTL_DEF_DRV(NVIDIA_GET_CRTC_CRC32,
                       nv_drm_get_crtc_crc32_ioctl,
                       DRM_RENDER_ALLOW|DRM_UNLOCKED),
+    DRM_IOCTL_DEF_DRV(NVIDIA_GET_CRTC_CRC32_V2,
+                      nv_drm_get_crtc_crc32_v2_ioctl,
+                      DRM_RENDER_ALLOW|DRM_UNLOCKED),
     DRM_IOCTL_DEF_DRV(NVIDIA_GEM_EXPORT_NVKMS_MEMORY,
                       nv_drm_gem_export_nvkms_memory_ioctl,
                       DRM_RENDER_ALLOW|DRM_UNLOCKED),
@@ -864,7 +840,7 @@ static void nv_drm_register_drm_device(const nv_gpu_info_t *gpu_info)
 
     if (nv_dev == NULL) {
         NV_DRM_LOG_ERR(
-            "Failed to allocate memmory for NVIDIA-DRM device object");
+            "Failed to allocate memory for NVIDIA-DRM device object");
         return;
     }
 
