@@ -26,7 +26,7 @@
 
 #include "nvtypes.h"
 #include "nvmisc.h"
-#include "uvmtypes.h"
+#include "uvm_types.h"
 #include "nv_uvm_types.h"
 #include "uvm_linux.h"
 #include "nv-kref.h"
@@ -493,26 +493,20 @@ typedef enum
     UVM_GPU_LINK_PCIE,
     UVM_GPU_LINK_NVLINK_1,
     UVM_GPU_LINK_NVLINK_2,
-
     UVM_GPU_LINK_NVLINK_3,
-
     UVM_GPU_LINK_MAX
 } uvm_gpu_link_type_t;
 
 // UVM does not support P2P copies on pre-Pascal GPUs. Pascal+ GPUs only
 // support virtual addresses in P2P copies. Therefore, a peer identity mapping
 // needs to be created.
-
 // Ampere+ GPUs support physical peer copies, too, so identity mappings are not
 // needed
-
 typedef enum
 {
     UVM_GPU_PEER_COPY_MODE_UNSUPPORTED,
     UVM_GPU_PEER_COPY_MODE_VIRTUAL,
-
     UVM_GPU_PEER_COPY_MODE_PHYSICAL,
-
     UVM_GPU_PEER_COPY_MODE_COUNT
 } uvm_gpu_peer_copy_mode_t;
 
@@ -602,11 +596,9 @@ struct uvm_gpu_struct
     struct
     {
         // Mask of peer_gpus set
-
         //
         // We can use a regular processor id because P2P is not allowed between
         // partitioned GPUs when SMC is enabled
-
         uvm_processor_mask_t peer_gpu_mask;
 
         // lazily-populated array of peer GPUs, indexed by the peer's GPU index
@@ -668,7 +660,6 @@ struct uvm_gpu_struct
         NvBool *error_notifier;
     } ecc;
 
-
     struct
     {
         NvU32 swizz_id;
@@ -679,7 +670,6 @@ struct uvm_gpu_struct
         uvmGpuDeviceHandle rm_device;
     } smc;
 
-
     struct
     {
         struct proc_dir_entry *dir;
@@ -819,10 +809,8 @@ struct uvm_parent_gpu_struct
 
     bool sparse_mappings_supported;
 
-
     // Ampere(GA100) requires map->invalidate->remap->invalidate for page size
     // promotion
-
     bool map_remap_larger_page_promotion;
 
     bool plc_supported;
@@ -950,7 +938,6 @@ struct uvm_parent_gpu_struct
     // This is set to true if the GPU belongs to an SLI group. Else, set to false.
     bool sli_enabled;
 
-
     struct
     {
         bool supported;
@@ -958,7 +945,6 @@ struct uvm_parent_gpu_struct
         bool enabled;
     } smc;
 
-
     // Global statistics. These fields are per-GPU and most of them are only
     // updated during fault servicing, and can be safely incremented.
     struct
@@ -1007,10 +993,8 @@ static const NvProcessorUuid *uvm_gpu_uuid(uvm_gpu_t *gpu)
 
 static uvmGpuDeviceHandle uvm_gpu_device_handle(uvm_gpu_t *gpu)
 {
-
     if (gpu->parent->smc.enabled)
         return gpu->smc.rm_device;
-
     return gpu->parent->rm_device;
 }
 
@@ -1115,11 +1099,9 @@ static uvm_gpu_phys_address_t uvm_gpu_page_to_phys_address(uvm_gpu_t *gpu, struc
 
 // Get a uvm_gpu_t by UUID.  This returns NULL if the GPU is not present.  This
 // is the general purpose call that should be used normally.
-
 // That is, unless a uvm_gpu_t for a specific SMC partition needs to be
 // retrieved, in which case uvm_gpu_get_by_uuid_and_swizz_id() must be used
 // instead.
-
 //
 // LOCKING: requires the global lock to be held
 uvm_gpu_t *uvm_gpu_get_by_uuid(const NvProcessorUuid *gpu_uuid);
@@ -1136,7 +1118,6 @@ uvm_parent_gpu_t *uvm_parent_gpu_get_by_uuid(const NvProcessorUuid *gpu_uuid);
 // limited cases.
 uvm_parent_gpu_t *uvm_parent_gpu_get_by_uuid_locked(const NvProcessorUuid *gpu_uuid);
 
-
 // Get the uvm_gpu_t for a partition by UUID and swizzId.  This returns NULL if
 // the partition hasn't been registered.  This call needs to be used instead
 // of uvm_gpu_get_by_uuid() when a specific partition is targeted.
@@ -1144,7 +1125,6 @@ uvm_parent_gpu_t *uvm_parent_gpu_get_by_uuid_locked(const NvProcessorUuid *gpu_u
 // LOCKING: requires the global lock to be held
 uvm_gpu_t *uvm_gpu_get_by_uuid_and_swizz_id(const NvProcessorUuid *gpu_uuid, NvU32 swizz_id);
 
-
 // Retain a gpu by uuid
 // Returns the retained uvm_gpu_t in gpu_out on success
 //
