@@ -24,7 +24,7 @@
 #ifndef __UVM_HAL_H__
 #define __UVM_HAL_H__
 
-#include "uvmtypes.h"
+#include "uvm_types.h"
 #include "uvm_common.h"
 #include "uvm_forward_decl.h"
 #include "uvm_hal_types.h"
@@ -172,10 +172,8 @@ void uvm_hal_pascal_ce_offset_in_out(uvm_push_t *push, NvU64 offset_in, NvU64 of
 
 typedef NvU32 (*uvm_hal_ce_phys_mode_t)(uvm_push_t *push, uvm_gpu_address_t dst, uvm_gpu_address_t src);
 NvU32 uvm_hal_kepler_ce_phys_mode(uvm_push_t *push, uvm_gpu_address_t dst, uvm_gpu_address_t src);
-
 NvU32 uvm_hal_ampere_ce_phys_mode(uvm_push_t *push, uvm_gpu_address_t dst, uvm_gpu_address_t src);
 
-
 typedef NvU32 (*uvm_hal_ce_plc_mode_t)(void);
 NvU32 uvm_hal_kepler_ce_plc_mode(void);
 
@@ -230,10 +228,11 @@ void uvm_hal_maxwell_arch_init_properties(uvm_parent_gpu_t *parent_gpu);
 void uvm_hal_pascal_arch_init_properties(uvm_parent_gpu_t *parent_gpu);
 void uvm_hal_volta_arch_init_properties(uvm_parent_gpu_t *parent_gpu);
 void uvm_hal_turing_arch_init_properties(uvm_parent_gpu_t *parent_gpu);
-
 void uvm_hal_ampere_arch_init_properties(uvm_parent_gpu_t *parent_gpu);
 
 
+
+
 // Retrieve the page-tree HAL for a given big page size
 typedef uvm_mmu_mode_hal_t *(*uvm_hal_lookup_mode_hal_t)(NvU32 big_page_size);
 typedef void (*uvm_hal_mmu_enable_prefetch_faults_t)(uvm_parent_gpu_t *parent_gpu);
@@ -242,9 +241,10 @@ uvm_mmu_mode_hal_t *uvm_hal_mmu_mode_kepler(NvU32 big_page_size);
 uvm_mmu_mode_hal_t *uvm_hal_mmu_mode_pascal(NvU32 big_page_size);
 uvm_mmu_mode_hal_t *uvm_hal_mmu_mode_volta(NvU32 big_page_size);
 uvm_mmu_mode_hal_t *uvm_hal_mmu_mode_turing(NvU32 big_page_size);
-
 uvm_mmu_mode_hal_t *uvm_hal_mmu_mode_ampere(NvU32 big_page_size);
 
+
+
 void uvm_hal_kepler_mmu_enable_prefetch_faults_unsupported(uvm_parent_gpu_t *parent_gpu);
 void uvm_hal_kepler_mmu_disable_prefetch_faults_unsupported(uvm_parent_gpu_t *parent_gpu);
 void uvm_hal_pascal_mmu_enable_prefetch_faults(uvm_parent_gpu_t *parent_gpu);
@@ -257,18 +257,20 @@ typedef uvm_mmu_engine_type_t (*uvm_hal_mmu_engine_id_to_type_t)(NvU16 mmu_engin
 uvm_mmu_engine_type_t uvm_hal_kepler_mmu_engine_id_to_type_unsupported(NvU16 mmu_engine_id);
 uvm_mmu_engine_type_t uvm_hal_volta_mmu_engine_id_to_type(NvU16 mmu_engine_id);
 uvm_mmu_engine_type_t uvm_hal_turing_mmu_engine_id_to_type(NvU16 mmu_engine_id);
-
 uvm_mmu_engine_type_t uvm_hal_ampere_mmu_engine_id_to_type(NvU16 mmu_engine_id);
 
 
+
+
 typedef NvU16 (*uvm_hal_mmu_client_id_to_utlb_id_t)(NvU16 client_id);
 NvU16 uvm_hal_kepler_mmu_client_id_to_utlb_id_unsupported(NvU16 client_id);
 NvU16 uvm_hal_pascal_mmu_client_id_to_utlb_id(NvU16 client_id);
 NvU16 uvm_hal_volta_mmu_client_id_to_utlb_id(NvU16 client_id);
-
 NvU16 uvm_hal_ampere_mmu_client_id_to_utlb_id(NvU16 client_id);
 
 
+
+
 // Replayable faults
 typedef void (*uvm_hal_enable_replayable_faults_t)(uvm_parent_gpu_t *parent_gpu);
 typedef void (*uvm_hal_disable_replayable_faults_t)(uvm_parent_gpu_t *parent_gpu);
@@ -351,7 +353,6 @@ void uvm_hal_volta_host_clear_faulted_channel_method(uvm_push_t *push,
 void uvm_hal_turing_host_clear_faulted_channel_method(uvm_push_t *push,
                                                       uvm_user_channel_t *user_channel,
                                                       const uvm_fault_buffer_entry_t *buffer_entry);
-
 typedef void (*uvm_hal_host_clear_faulted_channel_register_t)(uvm_user_channel_t *user_channel,
                                                               const uvm_fault_buffer_entry_t *buffer_entry);
 void uvm_hal_kepler_host_clear_faulted_channel_register_unsupported(uvm_user_channel_t *user_channel,
@@ -360,7 +361,6 @@ void uvm_hal_ampere_host_clear_faulted_channel_register(uvm_user_channel_t *user
                                                         const uvm_fault_buffer_entry_t *buffer_entry);
 
 
-
 void uvm_hal_print_fault_entry(const uvm_fault_buffer_entry_t *entry);
 void uvm_hal_print_access_counter_buffer_entry(const uvm_access_counter_buffer_entry_t *entry);
 
@@ -420,9 +420,7 @@ struct uvm_host_hal_struct
     uvm_hal_fault_cancel_targeted_t cancel_faults_targeted;
     uvm_hal_fault_cancel_va_t cancel_faults_va;
     uvm_hal_host_clear_faulted_channel_method_t clear_faulted_channel_method;
-
     uvm_hal_host_clear_faulted_channel_register_t clear_faulted_channel_register;
-
     uvm_hal_access_counter_clear_all_t access_counter_clear_all;
     uvm_hal_access_counter_clear_type_t access_counter_clear_type;
     uvm_hal_access_counter_clear_targeted_t access_counter_clear_targeted;
