@@ -21,7 +21,7 @@
 
 *******************************************************************************/
 
-#include "uvmtypes.h"
+#include "uvm_types.h"
 #include "uvm_forward_decl.h"
 #include "uvm_gpu.h"
 #include "uvm_mmu.h"
@@ -613,7 +613,6 @@ static NV_STATUS map_remap_init(uvm_page_tree_t *tree)
     if (status != NV_OK)
         goto error;
 
-
     // Allocate the pde0 struct (pde0 depth = depth(UVM_PAGE_SIZE_2M))
     // This is only needed for the 512M page size, since it requires a
     // PDE1-depth(512M) PTE. We first map it to the pde0 directory, then we
@@ -623,7 +622,6 @@ static NV_STATUS map_remap_init(uvm_page_tree_t *tree)
         if (status != NV_OK)
             goto error;
     }
-
     status = page_tree_begin_acquire(tree, &tree->tracker, &push, "map remap init");
     if (status != NV_OK)
         goto error;
@@ -640,7 +638,6 @@ static NV_STATUS map_remap_init(uvm_page_tree_t *tree)
 
     uvm_pte_batch_end(&batch);
 
-
     // Set pde0 entries to ptes_invalid_4k.
     if (tree->hal->page_sizes() & UVM_PAGE_SIZE_512M) {
         uvm_mmu_page_table_alloc_t *phys_allocs[2] = {NULL, NULL};
@@ -668,7 +665,6 @@ static NV_STATUS map_remap_init(uvm_page_tree_t *tree)
                  &push);
     }
 
-
     return page_tree_end_and_wait(tree, &push);
 error:
     map_remap_deinit(tree);
@@ -976,7 +972,6 @@ static NV_STATUS try_get_ptes(uvm_page_tree_t *tree,
 
 static NV_STATUS map_remap(uvm_page_tree_t *tree, NvU64 start, NvLength size, uvm_page_table_range_t *range)
 {
-
     NV_STATUS status;
     uvm_push_t push;
     NvU32 page_sizes;
@@ -1028,7 +1023,6 @@ static NV_STATUS map_remap(uvm_page_tree_t *tree, NvU64 start, NvLength size, uv
 
     page_tree_end(tree, &push);
     page_tree_tracker_overwrite_with_push(tree, &push);
-
     return NV_OK; 
 }
 
@@ -1795,13 +1789,11 @@ void uvm_mmu_init_gpu_chunk_sizes(uvm_parent_gpu_t *parent_gpu)
     // to handle allocating multiple chunks per page.
     parent_gpu->mmu_user_chunk_sizes = sizes & PAGE_MASK;
 
-
     // Ampere+ GPUs support 512MB page size, however, the maximum chunk size is
     // 2MB(i.e., UVM_CHUNK_SIZE_MAX), therefore we mask out any supported page
     // size greater than UVM_CHUNK_SIZE_MAX from the chunk size list.
     parent_gpu->mmu_user_chunk_sizes &= UVM_CHUNK_SIZES_MASK;
 
-
     parent_gpu->mmu_kernel_chunk_sizes = allocation_sizes_for_big_page_size(parent_gpu, UVM_PAGE_SIZE_64K) |
                                          allocation_sizes_for_big_page_size(parent_gpu, UVM_PAGE_SIZE_128K);
 }
