@@ -27,7 +27,7 @@
 #include "uvm_forward_decl.h"
 #include "uvm_hal_types.h"
 #include "uvm_pmm_gpu.h"
-#include "uvmtypes.h"
+#include "uvm_types.h"
 #include "uvm_common.h"
 #include "uvm_tracker.h"
 #include "uvm_test_ioctl.h"
@@ -35,8 +35,32 @@
 // Used when the page size isn't known and should not matter.
 #define UVM_PAGE_SIZE_AGNOSTIC 0
 
-// Memory layout of UVM's kernel VA space for Pascal+ GPUs.
+// Memory layout of UVM's kernel VA space.
 // The following memory regions are not to scale.
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+//
+// Pascal-Ampere:
 // +----------------+ 512TB
 // |                |
 // |   (not used)   |
@@ -249,10 +273,8 @@ struct uvm_page_tree_struct
 
         // PDE0 where all big-page entries are invalid, and small-page entries
         // point to ptes_invalid_4k.
-
         // pde0 is only used on Pascal-Ampere, i.e., they have the same PDE
         // format.
-
         uvm_mmu_page_table_alloc_t pde0;
     } map_remap;
 
