@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -107,7 +107,11 @@ static NV_STATUS test_push_inline_data_gpu(uvm_gpu_t *gpu)
             uvm_gpu_address_t data_gpu_address;
             char *inline_buf;
 
-            status = uvm_push_begin(gpu->channel_manager, UVM_CHANNEL_TYPE_GPU_INTERNAL, &push, "Inline data size %zu", test_size);
+            status = uvm_push_begin(gpu->channel_manager,
+                                    UVM_CHANNEL_TYPE_GPU_INTERNAL,
+                                    &push,
+                                    "Inline data size %zu",
+                                    test_size);
             TEST_CHECK_GOTO(status == NV_OK, done);
 
             // Do a noop first to test inline data starting at different offsets
@@ -260,7 +264,9 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     channel = uvm_channel_first(gpu->channel_manager);
     if (channel->num_gpfifo_entries <= TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES) {
         UVM_TEST_PRINT("Insufficient number of gpfifo entries per channel to run this test. Expected at least %u "
-                "entries, but found %u\n", TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES + 1, channel->num_gpfifo_entries);
+                       "entries, but found %u\n",
+                       TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES + 1,
+                       channel->num_gpfifo_entries);
         return NV_ERR_INVALID_STATE;
     }
     num_non_paused_pushes = channel->num_gpfifo_entries;
@@ -285,9 +291,9 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
         status = uvm_push_begin_on_channel(channel, pushes_not_ended + i, "Set to 0x%x", 0xDEADBEEF + i);
         TEST_CHECK_GOTO(status == NV_OK, done);
         gpu->parent->ce_hal->memset_v_4(pushes_not_ended + i,
-                                       gpu_va + sizeof(NvU32) * (i + 1),
-                                       0xDEADBEEF + i,
-                                       sizeof(NvU32));
+                                        gpu_va + sizeof(NvU32) * (i + 1),
+                                        0xDEADBEEF + i,
+                                        sizeof(NvU32));
 
         push_info = uvm_push_info_from_push(pushes_not_ended + i);
         push_info->on_complete = add_two_to_counter;
@@ -313,8 +319,9 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     for (i = 0; i < TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES; ++i)
         uvm_push_end(pushes_not_ended + i);
 
-    // When the channel manager becomes idle, the GPU methods have been completed, and the CPU completion callbacks
-    // associated with the push have been invoked.
+    // When the channel manager becomes idle, the GPU methods have been
+    // completed, and the CPU completion callbacks associated with the push
+    // have been invoked.
     status = uvm_channel_manager_wait(channel->pool->manager);
     TEST_CHECK_GOTO(status == NV_OK, done);
 
@@ -340,8 +347,8 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     expected = TEST_PUSH_INTERLEAVING_NUM_PAUSED_PUSHES * 2 + num_non_paused_pushes;
     if (observed != expected) {
         UVM_TEST_PRINT("Wrong value of counter incremented by push info callback. Observed %u but expected %u\n",
-                        observed,
-                        expected);
+                       observed,
+                       expected);
         status = NV_ERR_INVALID_STATE;
         goto done;
     }
@@ -353,18 +360,24 @@ done:
     return status;
 }
 
-// Using a single thread, interleave pushes and check that the result is consistent with a non-interleaved sequence.
-// 1) Begin a few pushes in channel X but do not end them. Each pushed (GPU) method sets a individual value in an
-//    independent system memory location.
-//    Each push is associated with a push info (CPU) callback that atomically adds 2 to a memory location M
-// 2) Begin and end many pushes in the same channel X such that all the gpfifo entries are filled. All the pushed
-//    methods do the same thing: atomically increment a given system memory location.
-//    Each push is associated with a push info callback that atomically increments the memory location M
+// Using a single thread, interleave pushes and check that the result is
+// consistent with a non-interleaved sequence.
+// 1) Begin a few pushes in channel X but do not end them. Each pushed (GPU)
+//    method sets a individual value in an independent system memory location.
+//    Each push is associated with a push info (CPU) callback that atomically
+//    adds 2 to a memory location M
+// 2) Begin and end many pushes in the same channel X such that all the gpfifo
+//    entries are filled. All the pushed methods do the same thing: atomically
+//    increment a given system memory location.
+//    Each push is associated with a push info callback that atomically
+//    increments the memory location M
 // 3) End the pending pushes
-// The final state should be the same as in the non-interleaved sequence (1)-(3)-(2)
 //
-// Starting more than a single push is not safe to do outside of a test as if multiple threads tried doing so,
-// it could easily deadlock.
+// The final state should be the same as in the non-interleaved sequence
+// (1)-(3)-(2)
+//
+// Starting more than a single push is not safe to do outside of a test as if
+// multiple threads tried doing so, it could easily deadlock.
 static NV_STATUS test_push_interleaving(uvm_va_space_t *va_space)
 {
     NV_STATUS status;
@@ -388,7 +401,10 @@ static NV_STATUS test_push_interleaving(uvm_va_space_t *va_space)
 // This is very tightly coupled with the pushbuffer implementation and method
 // sizes, which is not ideal, but allows to test corner cases in the pushbuffer
 // management code.
-static NV_STATUS test_push_exactly_max_push(uvm_gpu_t *gpu, uvm_push_t *push, uvm_gpu_semaphore_t *sema_to_acquire, NvU32 value)
+static NV_STATUS test_push_exactly_max_push(uvm_gpu_t *gpu,
+                                            uvm_push_t *push,
+                                            uvm_gpu_semaphore_t *sema_to_acquire,
+                                            NvU32 value)
 {
     NV_STATUS status;
 
@@ -549,7 +565,7 @@ static NV_STATUS test_idle_chunks_on_gpu(uvm_gpu_t *gpu)
 
         if (test_count_idle_chunks(gpu->channel_manager->pushbuffer) != UVM_PUSHBUFFER_CHUNKS - i - 1) {
             UVM_TEST_PRINT("Unexpected count of idle chunks in the pushbuffer %u instead of %u\n",
-                    test_count_idle_chunks(gpu->channel_manager->pushbuffer), UVM_PUSHBUFFER_CHUNKS - i - 1);
+                           test_count_idle_chunks(gpu->channel_manager->pushbuffer), UVM_PUSHBUFFER_CHUNKS - i - 1);
             uvm_pushbuffer_print(gpu->channel_manager->pushbuffer);
             status = NV_ERR_INVALID_STATE;
             goto done;
@@ -561,7 +577,8 @@ static NV_STATUS test_idle_chunks_on_gpu(uvm_gpu_t *gpu)
     TEST_CHECK_GOTO(status == NV_OK, done);
 
     if (test_count_idle_chunks(gpu->channel_manager->pushbuffer) != UVM_PUSHBUFFER_CHUNKS) {
-        UVM_TEST_PRINT("Unexpected count of idle chunks in the pushbuffer %u\n", test_count_idle_chunks(gpu->channel_manager->pushbuffer));
+        UVM_TEST_PRINT("Unexpected count of idle chunks in the pushbuffer %u\n",
+                       test_count_idle_chunks(gpu->channel_manager->pushbuffer));
         uvm_pushbuffer_print(gpu->channel_manager->pushbuffer);
         status = NV_ERR_INVALID_STATE;
         goto done;
@@ -834,9 +851,11 @@ NV_STATUS uvm_test_push_sanity(UVM_TEST_PUSH_SANITY_PARAMS *params, struct file
     if (status != NV_OK)
         goto done;
 
-    status = test_timestamp(va_space);
-    if (status != NV_OK)
-        goto done;
+    if (!params->skipTimestampTest) {
+        status = test_timestamp(va_space);
+        if (status != NV_OK)
+            goto done;
+    }
 
 done:
     uvm_va_space_up_read_rm(va_space);
