@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2019 NVIDIA Corporation
+    Copyright (c) 2016-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -52,11 +52,9 @@ static int resource_align_high_cmp(const void *a, const void *b)
     return 0;
 }
 
-static NV_STATUS get_rm_channel_resources(uvm_user_channel_t *user_channel)
+static NV_STATUS get_rm_channel_resources(uvm_user_channel_t *user_channel, UvmGpuChannelInstanceInfo *channel_info)
 {
     UvmGpuChannelResourceInfo *resources = NULL;
-    uvm_gpu_va_space_t *gpu_va_space = user_channel->gpu_va_space;
-    NV_STATUS status;
     NvU32 i, num_resources = user_channel->num_resources;
 
     // Note that num_resources may be 0, in which case resources will be
@@ -67,15 +65,7 @@ static NV_STATUS get_rm_channel_resources(uvm_user_channel_t *user_channel)
     if (!resources)
         return NV_ERR_NO_MEMORY;
 
-    status = uvm_rm_locked_call(nvUvmInterfaceRetainChannelResources(user_channel->rm_retained_channel, resources));
-    if (status != NV_OK) {
-        UVM_DBG_PRINT("Failed to get %u channel resources for {0x%x, 0x%x}: %s, GPU: %s\n",
-                      num_resources,
-                      user_channel->user_rm_channel.user_client, user_channel->user_rm_channel.user_object,
-                      nvstatusToString(status),
-                      uvm_gpu_name(gpu_va_space->gpu));
-        goto error;
-    }
+    memcpy(resources, channel_info->resourceInfo, num_resources * sizeof(resources[0]));
 
     // Info fix-up
     for (i = 0; i < num_resources; i++) {
@@ -97,10 +87,6 @@ static NV_STATUS get_rm_channel_resources(uvm_user_channel_t *user_channel)
 
     user_channel->resources = resources;
     return NV_OK;
-
-error:
-    uvm_kvfree(resources);
-    return status;
 }
 
 static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
@@ -110,7 +96,7 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
                                          NvU64 base,
                                          NvU64 length)
 {
-    UvmGpuChannelInstanceInfo channel_info;
+    UvmGpuChannelInstanceInfo *channel_info = NULL;
     uvm_user_channel_t *user_channel = NULL;
     NV_STATUS status = NV_OK;
     NvU32 rm_client = user_rm_channel->user_client;
@@ -145,13 +131,18 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
     //
     // TODO: Bug 1624521: This interface needs to use rm_control_fd to do
     //       validation.
-    memset(&channel_info, 0, sizeof(channel_info));
+    channel_info = uvm_kvmalloc_zero(sizeof(*channel_info));
+    if (!channel_info) {
+        status = NV_ERR_NO_MEMORY;
+        goto error;
+    }
+
     (void)user_channel->user_rm_channel.rm_control_fd;
     status = uvm_rm_locked_call(nvUvmInterfaceRetainChannel(user_channel->gpu_va_space->duped_gpu_va_space,
                                                             rm_client,
                                                             rm_channel,
                                                             &rm_retained_channel,
-                                                            &channel_info));
+                                                            channel_info));
     if (status != NV_OK) {
         UVM_DBG_PRINT("Failed to retain channel {0x%x, 0x%x}: %s, GPU: %s\n",
                       rm_client,
@@ -161,35 +152,33 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
         goto error;
     }
 
-    if (channel_info.sysmem)
+    if (channel_info->sysmem)
         user_channel->instance_ptr.addr.aperture = UVM_APERTURE_SYS;
     else
         user_channel->instance_ptr.addr.aperture = UVM_APERTURE_VID;
 
-    user_channel->instance_ptr.addr.address = channel_info.base;
-    user_channel->rm_retained_channel     = rm_retained_channel;
-    user_channel->hw_channel_id           = channel_info.chId;
-    user_channel->num_resources           = channel_info.resourceCount;
-    user_channel->engine_type             = channel_info.channelEngineType;
-    user_channel->in_subctx               = channel_info.bInSubctx == NV_TRUE;
-    user_channel->subctx_id               = channel_info.subctxId;
-    user_channel->tsg.valid               = channel_info.bTsgChannel == NV_TRUE;
-    user_channel->tsg.id                  = channel_info.tsgId;
-    user_channel->tsg.max_subctx_count    = channel_info.tsgMaxSubctxCount;
-    user_channel->work_submission_token   = channel_info.workSubmissionToken;
-    user_channel->work_submission_offset  = channel_info.workSubmissionOffset;
-    user_channel->clear_faulted_token     = channel_info.clearFaultedToken;
-
-    user_channel->chram_channel_register  = channel_info.pChramChannelRegister;
-    user_channel->smc_engine_id           = channel_info.smcEngineId;
-    user_channel->smc_engine_ve_id_offset = channel_info.smcEngineVeIdOffset;
+    user_channel->instance_ptr.addr.address = channel_info->base;
+    user_channel->rm_retained_channel       = rm_retained_channel;
+    user_channel->hw_channel_id             = channel_info->chId;
+    user_channel->num_resources             = channel_info->resourceCount;
+    user_channel->engine_type               = channel_info->channelEngineType;
+    user_channel->in_subctx                 = channel_info->bInSubctx == NV_TRUE;
+    user_channel->subctx_id                 = channel_info->subctxId;
+    user_channel->tsg.valid                 = channel_info->bTsgChannel == NV_TRUE;
+    user_channel->tsg.id                    = channel_info->tsgId;
+    user_channel->tsg.max_subctx_count      = channel_info->tsgMaxSubctxCount;
+    user_channel->work_submission_token     = channel_info->workSubmissionToken;
+    user_channel->work_submission_offset    = channel_info->workSubmissionOffset;
+    user_channel->clear_faulted_token       = channel_info->clearFaultedToken;
+    user_channel->chram_channel_register    = channel_info->pChramChannelRegister;
+    user_channel->smc_engine_id             = channel_info->smcEngineId;
+    user_channel->smc_engine_ve_id_offset   = channel_info->smcEngineVeIdOffset;
 
     if (!gpu->parent->smc.supported) {
         UVM_ASSERT(user_channel->smc_engine_id == 0);
         UVM_ASSERT(user_channel->smc_engine_ve_id_offset == 0);
     }
 
-
     // CE does not support subcontexts
     if (user_channel->in_subctx) {
         UVM_ASSERT(user_channel->engine_type != UVM_GPU_CHANNEL_ENGINE_TYPE_CE);
@@ -207,10 +196,12 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
         goto error;
     }
 
-    status = get_rm_channel_resources(user_channel);
+    status = get_rm_channel_resources(user_channel, channel_info);
     if (status != NV_OK)
         goto error;
 
+    uvm_kvfree(channel_info);
+
     *out_channel = user_channel;
     return NV_OK;
 
@@ -218,6 +209,7 @@ error:
     // uvm_user_channel_destroy_detached expects this
     user_channel->gpu_va_space = NULL;
     uvm_user_channel_destroy_detached(user_channel);
+    uvm_kvfree(channel_info);
     return status;
 }
 
@@ -355,6 +347,7 @@ static NV_STATUS create_va_range(uvm_user_channel_t *user_channel, NvU64 base, N
     range->channel.gpu_va_space     = gpu_va_space;
     range->channel.aperture         = aperture;
     range->channel.rm_descriptor    = resource->resourceDescriptor;
+    range->channel.rm_id            = resource->resourceId;
     range->channel.tsg.valid        = user_channel->tsg.valid;
     range->channel.tsg.id           = user_channel->tsg.id;
     range->channel.is_global        = resource->globalResource;
@@ -386,6 +379,7 @@ static void destroy_va_ranges(uvm_user_channel_t *user_channel)
 
         UVM_ASSERT(resource_range->type == UVM_VA_RANGE_TYPE_CHANNEL);
         UVM_ASSERT(resource_range->channel.rm_descriptor == user_channel->resources[i].resourceDescriptor);
+        UVM_ASSERT(resource_range->channel.rm_id         == user_channel->resources[i].resourceId);
         UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->tsg.valid);
         UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg.id);
 
@@ -448,10 +442,12 @@ static NV_STATUS bind_channel_resources(uvm_user_channel_t *user_channel)
         UVM_ASSERT(resource_range);
         UVM_ASSERT(resource_range->type == UVM_VA_RANGE_TYPE_CHANNEL);
         UVM_ASSERT(resource_range->channel.rm_descriptor == user_channel->resources[i].resourceDescriptor);
+        UVM_ASSERT(resource_range->channel.rm_id         == user_channel->resources[i].resourceId);
         UVM_ASSERT(resource_range->channel.tsg.valid == user_channel->tsg.valid);
         UVM_ASSERT(resource_range->channel.tsg.id == user_channel->tsg.id);
 
         resource_va_list[i].resourceDescriptor = resource_range->channel.rm_descriptor;
+        resource_va_list[i].resourceId         = resource_range->channel.rm_id;
         resource_va_list[i].resourceVa         = resource_range->node.start;
 
         // TODO: Bug 1732494: RM should be able to figure this out on its own
@@ -823,13 +819,8 @@ void uvm_user_channel_destroy_detached(uvm_user_channel_t *user_channel)
     uvm_tracker_wait_deinit(&user_channel->clear_faulted_tracker);
 
     if (user_channel->resources) {
-        NvU32 i;
         UVM_ASSERT(!user_channel->va_ranges);
 
-        for (i = 0; i < user_channel->num_resources; i++) {
-            NvP64 desc = user_channel->resources[i].resourceDescriptor;
-            uvm_rm_locked_call_void(nvUvmInterfaceReleaseChannelResources(&desc, 1));
-        }
         uvm_kvfree(user_channel->resources);
     }
 
@@ -944,21 +935,27 @@ out:
 
 NV_STATUS uvm_test_check_channel_va_space(UVM_TEST_CHECK_CHANNEL_VA_SPACE_PARAMS *params, struct file *filp)
 {
-    struct file *va_space_filp;
+    struct file *va_space_filp = NULL;
     uvm_va_space_t *va_space = NULL;
     uvm_va_space_t *channel_va_space;
     uvm_gpu_t *gpu;
     uvm_fault_buffer_entry_t fault_entry;
-    UvmGpuChannelInstanceInfo channel_info;
+    UvmGpuChannelInstanceInfo *channel_info;
     NV_STATUS status;
 
     memset(&fault_entry, 0, sizeof(fault_entry));
 
+    channel_info = uvm_kvmalloc_zero(sizeof(*channel_info));
+    if (!channel_info) {
+        status = NV_ERR_NO_MEMORY;
+        goto out;
+    }
+
     // The channel is owned by this file, so we have to query it using this
     // file's VA space.
-    status = uvm_test_check_channel_va_space_get_info(uvm_va_space_get(filp), params, &channel_info);
+    status = uvm_test_check_channel_va_space_get_info(uvm_va_space_get(filp), params, channel_info);
     if (status != NV_OK)
-        return status;
+        goto out;
 
     // We need to do the lookup using the input file's VA space
     va_space_filp = fget(params->va_space_fd);
@@ -990,20 +987,18 @@ NV_STATUS uvm_test_check_channel_va_space(UVM_TEST_CHECK_CHANNEL_VA_SPACE_PARAMS
     // Craft enough of the fault entry to do a VA space translation
     fault_entry.fault_type = UVM_FAULT_TYPE_INVALID_PTE;
 
-    if (channel_info.sysmem)
+    if (channel_info->sysmem)
         fault_entry.instance_ptr.aperture = UVM_APERTURE_SYS;
     else
         fault_entry.instance_ptr.aperture = UVM_APERTURE_VID;
-    fault_entry.instance_ptr.address = channel_info.base;
+    fault_entry.instance_ptr.address = channel_info->base;
 
-    if (channel_info.channelEngineType == UVM_GPU_CHANNEL_ENGINE_TYPE_GR) {
+    if (channel_info->channelEngineType == UVM_GPU_CHANNEL_ENGINE_TYPE_GR) {
         fault_entry.fault_source.client_type     = UVM_FAULT_CLIENT_TYPE_GPC;
         fault_entry.fault_source.mmu_engine_type = UVM_MMU_ENGINE_TYPE_GRAPHICS;
         fault_entry.fault_source.ve_id           = params->ve_id;
-
         // Translated to the SMC engine-local VEID
-        fault_entry.fault_source.ve_id += channel_info.smcEngineVeIdOffset;
-
+        fault_entry.fault_source.ve_id += channel_info->smcEngineVeIdOffset;
 
     }
     else if (gpu->parent->non_replayable_faults_supported) {
@@ -1034,5 +1029,7 @@ out:
         fput(va_space_filp);
     }
 
+    uvm_kvfree(channel_info);
+
     return status;
 }
