@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2019 NVIDIA Corporation
+    Copyright (c) 2015-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -278,6 +278,9 @@ typedef struct
     // their VA ranges.
     NvP64 rm_descriptor;
 
+    // This is an ID assigned by RM to each resource descriptor.
+    NvU32 rm_id;
+
     // The TSG which owns this mapping. Sharing of VA ranges is only allowed
     // within the same TSG. If valid == false, no sharing is allowed because the
     // channel is not in a TSG.
@@ -814,11 +817,14 @@ uvm_va_block_t *uvm_va_range_block_next(uvm_va_range_t *va_range, uvm_va_block_t
 // If mm != NULL, that mm is used for any CPU mappings which may be created as
 // a result of this call. See uvm_va_block_context_t::mm for details.
 //
+// If out_tracker != NULL any block work will be added to that tracker.
+//
 // LOCKING: If mm != NULL, the caller must hold mm->mmap_lock in at least read
 //          mode.
 NV_STATUS uvm_va_range_set_preferred_location(uvm_va_range_t *va_range,
                                               uvm_processor_id_t preferred_location,
-                                              struct mm_struct *mm);
+                                              struct mm_struct *mm,
+                                              uvm_tracker_t *out_tracker);
 
 // Add a processor to the accessed_by mask and establish any new required
 // mappings.
@@ -828,14 +834,23 @@ NV_STATUS uvm_va_range_set_preferred_location(uvm_va_range_t *va_range,
 // If mm != NULL, that mm is used for any CPU mappings which may be created as
 // a result of this call. See uvm_va_block_context_t::mm for details.
 //
+// If out_tracker != NULL any block work will be added to that tracker.
+//
 // LOCKING: If mm != NULL, the caller must hold mm->mmap_lock in at least read
 //          mode.
-NV_STATUS uvm_va_range_set_accessed_by(uvm_va_range_t *va_range, uvm_processor_id_t processor_id, struct mm_struct *mm);
+NV_STATUS uvm_va_range_set_accessed_by(uvm_va_range_t *va_range,
+                                       uvm_processor_id_t processor_id,
+                                       struct mm_struct *mm,
+                                       uvm_tracker_t *out_tracker);
 
 // Remove a processor from the accessed_by mask
 //
+// If out_tracker != NULL any block work will be added to that tracker.
+//
 // This also updates the mask and mappings of the UVM-Lite GPUs if required.
-void uvm_va_range_unset_accessed_by(uvm_va_range_t *va_range, uvm_processor_id_t processor_id);
+void uvm_va_range_unset_accessed_by(uvm_va_range_t *va_range,
+                                    uvm_processor_id_t processor_id,
+                                    uvm_tracker_t *out_tracker);
 
 // Set read-duplication and remove any existing accessed_by and remote mappings
 //
