@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2019 NVIDIA Corporation
+    Copyright (c) 2019-2020 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -24,6 +24,7 @@
 #include "nv-linux.h"
 #include "nv-caps.h"
 #include "nv-procfs.h"
+#include "nv-hash.h"
 
 /*
  * This temporary module parameter will be nuked soon when devfs support
@@ -41,103 +42,111 @@ extern int NVreg_ModifyDeviceFiles;
 
 #define NV_CAP_DRV_MINOR_COUNT 8192
 
+/* Hash table with 512 buckets */
+#define NV_CAP_HASH_BITS 9
+NV_DECLARE_HASHTABLE(g_nv_cap_hash_table, NV_CAP_HASH_BITS);
+
+#define NV_CAP_HASH_SIZE NV_HASH_SIZE(g_nv_cap_hash_table)
+
+#define nv_cap_hash_key(path) (nv_string_hash(path) % NV_CAP_HASH_SIZE)
+
 typedef struct nv_cap_table_entry
 {
+    /* name must be the first element */
     const char *name;
     int minor;
+    struct hlist_node hlist;
 } nv_cap_table_entry_t;
 
 #define NV_CAP_NUM_ENTRIES(_table) (sizeof(_table) / sizeof(_table[0]))
 
 static nv_cap_table_entry_t g_nv_cap_nvlink_table[] =
 {
-    { "fabric-mgmt", -1 }
+    {"/driver/nvidia-nvlink/capabilities/fabric-mgmt"}
 };
 
 static nv_cap_table_entry_t g_nv_cap_mig_table[] =
 {
-    { "config", -1 },
-    { "monitor", -1 }
+    {"/driver/nvidia/capabilities/mig/config"},
+    {"/driver/nvidia/capabilities/mig/monitor"}
 };
 
-#define NV_CAP_MIG_CI_ENTRIES(_gi) \
-    { _gi"/ci0/access", -1 },      \
-    { _gi"/ci1/access", -1 },      \
-    { _gi"/ci2/access", -1 },      \
-    { _gi"/ci3/access", -1 },      \
-    { _gi"/ci4/access", -1 },      \
-    { _gi"/ci5/access", -1 },      \
-    { _gi"/ci6/access", -1 },      \
-    { _gi"/ci7/access", -1 }
+#define NV_CAP_MIG_CI_ENTRIES(_gi)  \
+    {_gi "/ci0/access"},            \
+    {_gi "/ci1/access"},            \
+    {_gi "/ci2/access"},            \
+    {_gi "/ci3/access"},            \
+    {_gi "/ci4/access"},            \
+    {_gi "/ci5/access"},            \
+    {_gi "/ci6/access"},            \
+    {_gi "/ci7/access"}
 
-#define NV_CAP_MIG_GI_ENTRIES(_gpu) \
-    { "gi0/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi0"),   \
-    { "gi1/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi1"),   \
-    { "gi2/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi2"),   \
-    { "gi3/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi3"),   \
-    { "gi4/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi4"),   \
-    { "gi5/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi5"),   \
-    { "gi6/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi6"),   \
-    { "gi7/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi7"),   \
-    { "gi8/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi8"),   \
-    { "gi9/access", -1 },           \
-    NV_CAP_MIG_CI_ENTRIES("gi9"),   \
-    { "gi10/access", -1 },          \
-    NV_CAP_MIG_CI_ENTRIES("gi10"),  \
-    { "gi11/access", -1 },          \
-    NV_CAP_MIG_CI_ENTRIES("gi11"),  \
-    { "gi12/access", -1 },          \
-    NV_CAP_MIG_CI_ENTRIES("gi12"),  \
-    { "gi13/access", -1 },          \
-    NV_CAP_MIG_CI_ENTRIES("gi13"),  \
-    { "gi14/access", -1 },          \
-    NV_CAP_MIG_CI_ENTRIES("gi14")
-
-#define NV_CAP_MAX_GPUS 32
+#define NV_CAP_MIG_GI_ENTRIES(_gpu)       \
+    {_gpu "/gi0/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi0"),   \
+    {_gpu "/gi1/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi1"),   \
+    {_gpu "/gi2/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi2"),   \
+    {_gpu "/gi3/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi3"),   \
+    {_gpu "/gi4/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi4"),   \
+    {_gpu "/gi5/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi5"),   \
+    {_gpu "/gi6/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi6"),   \
+    {_gpu "/gi7/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi7"),   \
+    {_gpu "/gi8/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi8"),   \
+    {_gpu "/gi9/access"},                 \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi9"),   \
+    {_gpu "/gi10/access"},                \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi10"),  \
+    {_gpu "/gi11/access"},                \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi11"),  \
+    {_gpu "/gi12/access"},                \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi12"),  \
+    {_gpu "/gi13/access"},                \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi13"),  \
+    {_gpu "/gi14/access"},                \
+    NV_CAP_MIG_CI_ENTRIES(_gpu "/gi14")
 
 static nv_cap_table_entry_t g_nv_cap_mig_gpu_table[] =
 {
-    NV_CAP_MIG_GI_ENTRIES("gpu0"),
-    NV_CAP_MIG_GI_ENTRIES("gpu1"),
-    NV_CAP_MIG_GI_ENTRIES("gpu2"),
-    NV_CAP_MIG_GI_ENTRIES("gpu3"),
-    NV_CAP_MIG_GI_ENTRIES("gpu4"),
-    NV_CAP_MIG_GI_ENTRIES("gpu5"),
-    NV_CAP_MIG_GI_ENTRIES("gpu6"),
-    NV_CAP_MIG_GI_ENTRIES("gpu7"),
-    NV_CAP_MIG_GI_ENTRIES("gpu8"),
-    NV_CAP_MIG_GI_ENTRIES("gpu9"),
-    NV_CAP_MIG_GI_ENTRIES("gpu10"),
-    NV_CAP_MIG_GI_ENTRIES("gpu11"),
-    NV_CAP_MIG_GI_ENTRIES("gpu12"),
-    NV_CAP_MIG_GI_ENTRIES("gpu13"),
-    NV_CAP_MIG_GI_ENTRIES("gpu14"),
-    NV_CAP_MIG_GI_ENTRIES("gpu15"),
-    NV_CAP_MIG_GI_ENTRIES("gpu16"),
-    NV_CAP_MIG_GI_ENTRIES("gpu17"),
-    NV_CAP_MIG_GI_ENTRIES("gpu18"),
-    NV_CAP_MIG_GI_ENTRIES("gpu19"),
-    NV_CAP_MIG_GI_ENTRIES("gpu20"),
-    NV_CAP_MIG_GI_ENTRIES("gpu21"),
-    NV_CAP_MIG_GI_ENTRIES("gpu22"),
-    NV_CAP_MIG_GI_ENTRIES("gpu23"),
-    NV_CAP_MIG_GI_ENTRIES("gpu24"),
-    NV_CAP_MIG_GI_ENTRIES("gpu25"),
-    NV_CAP_MIG_GI_ENTRIES("gpu26"),
-    NV_CAP_MIG_GI_ENTRIES("gpu27"),
-    NV_CAP_MIG_GI_ENTRIES("gpu28"),
-    NV_CAP_MIG_GI_ENTRIES("gpu29"),
-    NV_CAP_MIG_GI_ENTRIES("gpu30"),
-    NV_CAP_MIG_GI_ENTRIES("gpu31")
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu0/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu1/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu2/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu3/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu4/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu5/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu6/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu7/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu8/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu9/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu10/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu11/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu12/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu13/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu14/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu15/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu16/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu17/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu18/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu19/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu20/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu21/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu22/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu23/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu24/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu25/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu26/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu27/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu28/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu29/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu30/mig"),
+    NV_CAP_MIG_GI_ENTRIES("/driver/nvidia/capabilities/gpu31/mig")
 };
 
 struct nv_cap
@@ -172,6 +181,7 @@ struct
 } g_nv_cap_drv;
 
 #define NV_CAP_PROCFS_DIR "driver/nvidia-caps"
+#define NV_CAP_NAME_BUF_SIZE 128
 
 static struct proc_dir_entry *nv_cap_procfs_dir;
 static struct proc_dir_entry *nv_cap_procfs_nvlink_minors;
@@ -180,12 +190,17 @@ static struct proc_dir_entry *nv_cap_procfs_mig_minors;
 static int nv_procfs_read_nvlink_minors(struct seq_file *s, void *v)
 {
     int i, count;
+    char name[NV_CAP_NAME_BUF_SIZE];
 
     count = NV_CAP_NUM_ENTRIES(g_nv_cap_nvlink_table);
     for (i = 0; i < count; i++)
     {
-        seq_printf(s, "%s %d\n", g_nv_cap_nvlink_table[i].name,
-                   g_nv_cap_nvlink_table[i].minor);
+        if (sscanf(g_nv_cap_nvlink_table[i].name,
+                   "/driver/nvidia-nvlink/capabilities/%s", name) == 1)
+        {
+            name[sizeof(name) - 1] = '\0';
+            seq_printf(s, "%s %d\n", name, g_nv_cap_nvlink_table[i].minor);
+        }
     }
 
     return 0;
@@ -193,25 +208,29 @@ static int nv_procfs_read_nvlink_minors(struct seq_file *s, void *v)
 
 static int nv_procfs_read_mig_minors(struct seq_file *s, void *v)
 {
-    int i, j, count, index;
+    int i, count, gpu;
+    char name[NV_CAP_NAME_BUF_SIZE];
 
     count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_table);
     for (i = 0; i < count; i++)
     {
-        seq_printf(s, "%s %d\n", g_nv_cap_mig_table[i].name,
-                   g_nv_cap_mig_table[i].minor);
+        if (sscanf(g_nv_cap_mig_table[i].name,
+                   "/driver/nvidia/capabilities/mig/%s", name) == 1)
+        {
+            name[sizeof(name) - 1] = '\0';
+            seq_printf(s, "%s %d\n", name, g_nv_cap_mig_table[i].minor);
+        }
     }
 
-    index = 0;
-    count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_gpu_table) / NV_CAP_MAX_GPUS;
-    for (i = 0; i < NV_CAP_MAX_GPUS; i++)
+    count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_gpu_table);
+    for (i = 0; i < count; i++)
     {
-        for (j = 0; j < count; j++)
+        if (sscanf(g_nv_cap_mig_gpu_table[i].name,
+                   "/driver/nvidia/capabilities/gpu%d/mig/%s", &gpu, name) == 2)
         {
-            seq_printf(s, "gpu%d/%s %d\n", i,
-                       g_nv_cap_mig_gpu_table[index].name,
-                       g_nv_cap_mig_gpu_table[index].minor);
-            index++;
+            name[sizeof(name) - 1] = '\0';
+            seq_printf(s, "gpu%d/%s %d\n",
+                       gpu, name, g_nv_cap_mig_gpu_table[i].minor);
         }
     }
 
@@ -271,114 +290,53 @@ cleanup:
     return -EACCES;
 }
 
-static int nv_cap_find_minor_in_table(nv_cap_table_entry_t *table, int count,
-                                      const char *target)
-{
-    int i;
-
-    for (i = 0; i < count; i++)
-    {
-        if (strcmp(table[i].name, target) == 0)
-        {
-            return table[i].minor;
-        }
-    }
-
-    return -1;
-}
-
 static int nv_cap_find_minor(char *path)
 {
-    char *target;
-    int minor;
-    unsigned int len, count, gpu, index;
+    unsigned int key = nv_cap_hash_key(path);
+    nv_cap_table_entry_t *entry;
 
-    target = "/driver/nvidia-nvlink/capabilities/";
-    len = strlen(target);
-
-    if (strncmp(path, target, len) == 0)
+    nv_hash_for_each_possible(g_nv_cap_hash_table, entry, hlist, key)
     {
-        target = path + len;
-        count = NV_CAP_NUM_ENTRIES(g_nv_cap_nvlink_table);
-        minor = nv_cap_find_minor_in_table(g_nv_cap_nvlink_table,
-                                           count, target);
-        if (minor >= 0)
+        if (strcmp(path, entry->name) == 0)
         {
-            return minor;
-        }
-    }
-
-    target = "/driver/nvidia/capabilities/gpu";
-    len = strlen(target);
-
-    if (strncmp(path, target, len) == 0)
-    {
-        target = path + len;
-        sscanf(target, "%u\n", &gpu);
-        target = strchr(target, '/');
-        WARN_ON(gpu >= NV_CAP_MAX_GPUS);
-        WARN_ON(target == NULL);
-
-        len = strlen("/mig/");
-        if (strncmp(target, "/mig/", len) == 0)
-        {
-            target = target + len;
-            count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_gpu_table) / NV_CAP_MAX_GPUS;
-            index = gpu * count;
-            minor = nv_cap_find_minor_in_table(&g_nv_cap_mig_gpu_table[index],
-                                               count, target);
-            if (minor >= 0)
-            {
-                return minor;
-            }
-        }
-    }
-
-    target = "/driver/nvidia/capabilities/mig/";
-    len = strlen(target);
-
-    if (strncmp(path, target, len) == 0)
-    {
-        target = path + len;
-
-        count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_table);
-        minor = nv_cap_find_minor_in_table(g_nv_cap_mig_table, count, target);
-        if (minor >= 0)
-        {
-            return minor;
+            return entry->minor;
         }
     }
 
     return -1;
 }
 
-static void nv_cap_assign_minors(void)
+static void _nv_cap_table_init(nv_cap_table_entry_t *table, int count)
 {
-    int minor = 0;
-    int count;
     int i;
+    unsigned int key;
+    static int minor = 0;
 
-    count = NV_CAP_NUM_ENTRIES(g_nv_cap_nvlink_table);
     for (i = 0; i < count; i++)
     {
-        g_nv_cap_nvlink_table[i].minor = minor++;
-    }
-
-    count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_table);
-    for (i = 0; i < count; i++)
-    {
-        g_nv_cap_mig_table[i].minor = minor++;
-    }
-
-    count = NV_CAP_NUM_ENTRIES(g_nv_cap_mig_gpu_table);
-    for (i = 0; i < count; i++)
-    {
-        g_nv_cap_mig_gpu_table[i].minor = minor++;
+        table[i].minor = minor++;
+        INIT_HLIST_NODE(&table[i].hlist);
+        key = nv_cap_hash_key(table[i].name);
+        nv_hash_add(g_nv_cap_hash_table, &table[i].hlist, key);
     }
 
     WARN_ON(minor > NV_CAP_DRV_MINOR_COUNT);
 }
 
+#define nv_cap_table_init(table) \
+    _nv_cap_table_init(table, NV_CAP_NUM_ENTRIES(table))
+
+static void nv_cap_tables_init(void)
+{
+    BUILD_BUG_ON(offsetof(nv_cap_table_entry_t, name) != 0);
+
+    nv_hash_init(g_nv_cap_hash_table);
+
+    nv_cap_table_init(g_nv_cap_nvlink_table);
+    nv_cap_table_init(g_nv_cap_mig_table);
+    nv_cap_table_init(g_nv_cap_mig_gpu_table);
+}
+
 static ssize_t nv_cap_procfs_write(struct file *file,
                                     const char __user *buffer,
                                     size_t count, loff_t *pos)
@@ -856,7 +814,7 @@ int NV_API_CALL nv_cap_drv_init(void)
 {
     int rc;
 
-    nv_cap_assign_minors();
+    nv_cap_tables_init();
 
     if (!nv_cap_enable_devfs)
     {
