@@ -15,8 +15,6 @@
 #include "nv-linux.h"
 #include "nv_speculation_barrier.h"
 
-extern nv_cpu_type_t nv_cpu_type;
-
 /*
  * The 'struct vm_operations' open() callback is called by the Linux
  * kernel when the parent VMA is split or copied, close() when the
@@ -294,12 +292,7 @@ int nv_encode_caching(
             break;
 #endif
         case NV_MEMORY_UNCACHED:
-            /*!
-             * On Tegra 3 (A9), we cannot have the device type bits set on
-             * any BAR mappings.
-             */
-            *prot = ((memory_type == NV_MEMORY_TYPE_SYSTEM) ||
-                (nv_cpu_type == NV_CPU_TYPE_ARM_A9)) ?
+            *prot = (memory_type == NV_MEMORY_TYPE_SYSTEM) ?
                     NV_PGPROT_UNCACHED(*prot) :
                     NV_PGPROT_UNCACHED_DEVICE(*prot);
             break;
@@ -308,12 +301,7 @@ int nv_encode_caching(
         case NV_MEMORY_WRITECOMBINED:
             if (NV_ALLOW_WRITE_COMBINING(memory_type))
             {
-                /*!
-                 * On Tegra 3 (A9), we cannot have the device type bits set on
-                 * any BAR mappings.
-                 */
-                *prot = ((memory_type == NV_MEMORY_TYPE_FRAMEBUFFER) &&
-                    (nv_cpu_type != NV_CPU_TYPE_ARM_A9)) ?
+                *prot = (memory_type == NV_MEMORY_TYPE_FRAMEBUFFER) ?
                         NV_PGPROT_WRITE_COMBINED_DEVICE(*prot) :
                         NV_PGPROT_WRITE_COMBINED(*prot);
                 break;
@@ -333,6 +321,7 @@ int nv_encode_caching(
         case NV_MEMORY_CACHED:
             if (NV_ALLOW_CACHING(memory_type))
                 break;
+            // Intentional fallthrough.
         default:
             nv_printf(NV_DBG_ERRORS,
                 "NVRM: VM: cache type %d not supported for memory type %d!\n",
@@ -521,7 +510,8 @@ int nvidia_mmap_helper(
             else
             {
                 if (nv_encode_caching(&vma->vm_page_prot,
-                        NV_MEMORY_WRITECOMBINED, NV_MEMORY_TYPE_FRAMEBUFFER))
+                        rm_disable_iomap_wc() ? NV_MEMORY_UNCACHED : NV_MEMORY_WRITECOMBINED, 
+                        NV_MEMORY_TYPE_FRAMEBUFFER))
                 {
                     if (nv_encode_caching(&vma->vm_page_prot,
                             NV_MEMORY_UNCACHED_WEAK, NV_MEMORY_TYPE_FRAMEBUFFER))
@@ -662,9 +652,9 @@ int nvidia_mmap(
 
     //
     // Do not allow mmap operation if this is a fd into
-    // which a rm-object has been exported.
+    // which rm objects have been exported.
     //
-    if (nvlfp->nvfp.numExportedRmObjects != 0)
+    if (nvlfp->nvfp.handles != NULL)
     {
         return -EINVAL;
     }
