@@ -20,32 +20,8 @@
 #include "nv-vgpu-vfio-interface.h"
 #endif
 
-nv_cpu_type_t nv_cpu_type = NV_CPU_TYPE_UNKNOWN;
-
-nv_pci_info_t nv_assign_gpu_pci_info[NV_MAX_DEVICES];
-
 static void
-nv_pci_validate_assigned_gpus(struct pci_dev *pci_dev)
-{
-    NvU32 i;
-
-    if (NV_IS_ASSIGN_GPU_PCI_INFO_SPECIFIED())
-    {
-        for (i = 0; i < nv_assign_gpu_count; i++)
-        {
-            if ((nv_assign_gpu_pci_info[i].domain == NV_PCI_DOMAIN_NUMBER(pci_dev)) &&
-                (nv_assign_gpu_pci_info[i].bus == NV_PCI_BUS_NUMBER(pci_dev)) &&
-                (nv_assign_gpu_pci_info[i].slot == NV_PCI_SLOT_NUMBER(pci_dev)))
-            {
-                nv_assign_gpu_pci_info[i].valid = NV_TRUE;
-                return;
-            }
-        }
-    }
-}
-
-static void
-nv_check_and_blacklist_gpu(
+nv_check_and_exclude_gpu(
     nvidia_stack_t *sp,
     nv_state_t *nv
 )
@@ -59,17 +35,17 @@ nv_check_and_blacklist_gpu(
         return;
     }
 
-    if (nv_is_uuid_in_gpu_blacklist(uuid_str))
+    if (nv_is_uuid_in_gpu_exclusion_list(uuid_str))
     {
-        NV_STATUS rm_status = rm_blacklist_adapter(sp, nv);
+        NV_STATUS rm_status = rm_exclude_adapter(sp, nv);
         if (rm_status != NV_OK)
         {
             NV_DEV_PRINTF_STATUS(NV_DBG_ERRORS, nv, rm_status,
-                          "Failed to blacklist %s", uuid_str);
+                          "Failed to exclude GPU %s", uuid_str);
             goto done;
         }
-        nv->flags |= NV_FLAG_BLACKLIST;
-        NV_DEV_PRINTF(NV_DBG_INFO, nv, "Blacklisted %s successfully\n",
+        nv->flags |= NV_FLAG_EXCLUDE;
+        NV_DEV_PRINTF(NV_DBG_INFO, nv, "Excluded GPU %s successfully\n",
                       uuid_str);
     }
 
@@ -194,11 +170,14 @@ nv_pci_probe
 #endif /* NV_PCI_SRIOV_SUPPORT */
 
 
-    if ((pci_dev->vendor != 0x10de) || (pci_dev->device < 0x20) ||
-        ((pci_dev->class != (PCI_CLASS_DISPLAY_VGA << 8)) &&
-         (pci_dev->class != (PCI_CLASS_DISPLAY_3D << 8))) ||
-        rm_is_legacy_device(pci_dev->device, pci_dev->subsystem_vendor,
-                            pci_dev->subsystem_device, NV_FALSE))
+    if (!rm_is_supported_pci_device(
+                (pci_dev->class >> 16) & 0xFF,
+                (pci_dev->class >> 8) & 0xFF,
+                pci_dev->vendor,
+                pci_dev->device,
+                pci_dev->subsystem_vendor,
+                pci_dev->subsystem_device,
+                NV_FALSE /* print_legacy_warning */))
     {
         nv_printf(NV_DBG_ERRORS, "NVRM: ignoring the legacy GPU %04x:%02x:%02x.%x\n",
                   NV_PCI_DOMAIN_NUMBER(pci_dev), NV_PCI_BUS_NUMBER(pci_dev),
@@ -206,23 +185,6 @@ nv_pci_probe
         goto failed;
     }
 
-    if (NV_IS_ASSIGN_GPU_PCI_INFO_SPECIFIED())
-    {
-        for (i = 0; i < nv_assign_gpu_count; i++)
-        {
-            if (((nv_assign_gpu_pci_info[i].domain == NV_PCI_DOMAIN_NUMBER(pci_dev)) &&
-                 (nv_assign_gpu_pci_info[i].bus == NV_PCI_BUS_NUMBER(pci_dev)) &&
-                 (nv_assign_gpu_pci_info[i].slot == NV_PCI_SLOT_NUMBER(pci_dev))) &&
-                (nv_assign_gpu_pci_info[i].valid))
-                break;
-        }
-
-        if (i == nv_assign_gpu_count)
-        {
-            goto failed;
-        }
-    }
-
     num_probed_nv_devices++;
 
     if (pci_enable_device(pci_dev) != 0)
@@ -242,7 +204,7 @@ nv_pci_probe
         goto failed;
     }
 
-    for (i = 0; i < (NV_GPU_NUM_BARS - 1); i++)
+    for (i = 0, j = 0; i < NVRM_PCICFG_NUM_BARS && j < NV_GPU_NUM_BARS; i++)
     {
         if (NV_PCI_RESOURCE_VALID(pci_dev, i))
         {
@@ -272,12 +234,12 @@ nv_pci_probe
                 NvU32 base_upper, limit_upper;
 
                 if (bridge == NULL)
-                    continue;
+                    goto next_bar;
 
                 pci_read_config_dword(pci_dev, NVRM_PCICFG_BAR_OFFSET(i) + 4,
-                        &base_upper);
+                                      &base_upper);
                 if (base_upper == 0)
-                    continue;
+                    goto next_bar;
 
                 pci_read_config_dword(bridge, PCI_PREF_BASE_UPPER32,
                         &base_upper);
@@ -285,7 +247,7 @@ nv_pci_probe
                         &limit_upper);
 
                 if ((base_upper != 0) && (limit_upper != 0))
-                    continue;
+                    goto next_bar;
 
                 nv_printf(NV_DBG_ERRORS,
                     "NVRM: This is a 64-bit BAR mapped above 4GB by the system\n"
@@ -299,41 +261,11 @@ nv_pci_probe
                     "NVRM: information.\n");
                 goto failed;
             }
-                continue;
+
+next_bar:
+            j++;
+            continue;
         }
-        nv_printf(NV_DBG_ERRORS,
-            "NVRM: This PCI I/O region assigned to your NVIDIA device is invalid:\n"
-            "NVRM: BAR%d is %dM @ 0x%llx (PCI:%04x:%02x:%02x.%x)\n", i,
-            (NV_PCI_RESOURCE_SIZE(pci_dev, i) >> 20),
-            (NvU64)NV_PCI_RESOURCE_START(pci_dev, i),
-            NV_PCI_DOMAIN_NUMBER(pci_dev), NV_PCI_BUS_NUMBER(pci_dev),
-            NV_PCI_SLOT_NUMBER(pci_dev), PCI_FUNC(pci_dev->devfn));
-#if defined(NVCPU_X86)
-        if ((NV_PCI_RESOURCE_FLAGS(pci_dev, i) & PCI_BASE_ADDRESS_MEM_TYPE_64) &&
-            ((NV_PCI_RESOURCE_START(pci_dev, i) >> PAGE_SHIFT) > 0xfffffULL))
-        {
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: This is a 64-bit BAR mapped above 4GB by the system\n"
-                "NVRM: BIOS or the Linux kernel.  The NVIDIA Linux/x86\n"
-                "NVRM: graphics driver and other system software components\n"
-                "NVRM: do not support this configuration.\n");
-        }
-        else
-#endif
-        if (NV_PCI_RESOURCE_FLAGS(pci_dev, i) & PCI_BASE_ADDRESS_MEM_TYPE_64)
-        {
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: This is a 64-bit BAR, which some operating system\n"
-                "NVRM: kernels and other system software components are known\n"
-                "NVRM: to handle incorrectly.  Please see the README section\n"
-                "NVRM: on 64-bit BARs for more information.\n");
-        }
-        else
-        {
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: The system BIOS may have misconfigured your GPU.\n");
-        }
-        goto failed;
     }
 
     if (!request_mem_region(NV_PCI_RESOURCE_START(pci_dev, NV_GPU_BAR_INDEX_REGS),
@@ -428,9 +360,6 @@ nv_pci_probe
     vga_set_legacy_decoding(pci_dev, VGA_RSRC_NONE);
 #endif
 
-    if (rm_get_cpu_type(sp, &nv_cpu_type) != NV_OK)
-        nv_printf(NV_DBG_ERRORS, "NVRM: error retrieving cpu type\n");
-
     status = nv_check_gpu_state(nv);
     if (status == NV_ERR_GPU_IS_LOST)
     {
@@ -460,25 +389,6 @@ nv_pci_probe
 
     num_nv_devices++;
 
-    for (i = 0; i < NV_GPU_NUM_BARS; i++)
-    {
-        if (nv->bars[i].size != 0)
-        {
-            if (nv_user_map_register(nv->bars[i].cpu_address,
-                nv->bars[i].strapped_size) != 0)
-            {
-                NV_DEV_PRINTF(NV_DBG_ERRORS, nv,
-                              "Failed to register usermap for BAR %u!\n", i);
-                for (j = 0; j < i; j++)
-                {
-                    nv_user_map_unregister(nv->bars[j].cpu_address,
-                                           nv->bars[j].strapped_size);
-                }
-                goto err_zero_dev;
-            }
-        }
-    }
-
     /*
      * The newly created nvl object is added to the nv_linux_devices global list
      * only after all the initialization operations for that nvl object are
@@ -512,7 +422,7 @@ nv_pci_probe
     }
 #endif
 
-    nv_check_and_blacklist_gpu(sp, nv);
+    nv_check_and_exclude_gpu(sp, nv);
 
     nv_init_dynamic_power_management(sp, pci_dev);
 
@@ -553,7 +463,6 @@ nv_pci_remove(struct pci_dev *pci_dev)
     nv_linux_state_t *nvl = NULL;
     nv_state_t *nv;
     nvidia_stack_t *sp = NULL;
-    NvU32 i;
 
     nv_printf(NV_DBG_SETUP, "NVRM: removing GPU %04x:%02x:%02x.%x\n",
               NV_PCI_DOMAIN_NUMBER(pci_dev), NV_PCI_BUS_NUMBER(pci_dev),
@@ -564,7 +473,8 @@ nv_pci_remove(struct pci_dev *pci_dev)
     if (pci_dev->is_virtfn)
     {
 #if defined(NV_VGPU_KVM_BUILD)
-        nvidia_vgpu_vfio_remove(pci_dev);
+        /* Arg 2 == NV_TRUE means that the PCI device should be removed */
+        nvidia_vgpu_vfio_remove(pci_dev, NV_TRUE);
 #endif /* NV_VGPU_KVM_BUILD */
         return;
     }
@@ -652,7 +562,8 @@ nv_pci_remove(struct pci_dev *pci_dev)
 #endif
 
 #if defined(NV_VGPU_KVM_BUILD)
-    nvidia_vgpu_vfio_remove(pci_dev);
+    /* Arg 2 == NV_TRUE means that the PCI device should be removed */
+    nvidia_vgpu_vfio_remove(pci_dev, NV_TRUE);
 #endif
 
     /* Update the frontend data structures */
@@ -663,6 +574,7 @@ nv_pci_remove(struct pci_dev *pci_dev)
 
     if ((nv->flags & NV_FLAG_PERSISTENT_SW_STATE) || (nv->flags & NV_FLAG_OPEN))
     {
+        nv_acpi_unregister_notifier(nvl);
         if (nv->flags & NV_FLAG_PERSISTENT_SW_STATE)
         {
             rm_disable_gpu_state_persistence(sp, nv);
@@ -689,14 +601,6 @@ nv_pci_remove(struct pci_dev *pci_dev)
 
     pci_set_drvdata(pci_dev, NULL);
 
-    for (i = 0; i < NV_GPU_NUM_BARS; i++)
-    {
-        if (nv->bars[i].size != 0)
-        {
-            nv_user_map_unregister(nv->bars[i].cpu_address,
-                                   nv->bars[i].strapped_size);
-        }
-    }
     rm_i2c_remove_adapters(sp, nv);
     rm_free_private_state(sp, nv);
     release_mem_region(NV_PCI_RESOURCE_START(pci_dev, NV_GPU_BAR_INDEX_REGS),
@@ -844,41 +748,38 @@ nv_pci_count_devices(void)
         return 0;
     }
 
-    pci_dev = NV_PCI_GET_CLASS(PCI_CLASS_DISPLAY_VGA << 8, NULL);
+    pci_dev = pci_get_class(PCI_CLASS_DISPLAY_VGA << 8, NULL);
     while (pci_dev)
     {
-        if ((pci_dev->vendor == 0x10de) && (pci_dev->device >= 0x20) &&
-            !rm_is_legacy_device(pci_dev->device, pci_dev->subsystem_vendor,
-                                 pci_dev->subsystem_device, NV_TRUE))
+        if (rm_is_supported_pci_device(
+                PCI_BASE_CLASS_DISPLAY,
+                PCI_CLASS_DISPLAY_VGA & 0xFF,
+                pci_dev->vendor,
+                pci_dev->device,
+                pci_dev->subsystem_vendor,
+                pci_dev->subsystem_device,
+                NV_TRUE /* print_legacy_warning */))
         {
             count++;
-            nv_pci_validate_assigned_gpus(pci_dev);
         }
-        pci_dev = NV_PCI_GET_CLASS(PCI_CLASS_DISPLAY_VGA << 8, pci_dev);
+        pci_dev = pci_get_class(PCI_CLASS_DISPLAY_VGA << 8, pci_dev);
     }
 
-    pci_dev = NV_PCI_GET_CLASS(PCI_CLASS_DISPLAY_3D << 8, NULL);
+    pci_dev = pci_get_class(PCI_CLASS_DISPLAY_3D << 8, NULL);
     while (pci_dev)
     {
-        if ((pci_dev->vendor == 0x10de) && (pci_dev->device >= 0x20) &&
-            !rm_is_legacy_device(pci_dev->device, pci_dev->subsystem_vendor,
-                                 pci_dev->subsystem_device, NV_TRUE))
+        if (rm_is_supported_pci_device(
+                (pci_dev->class >> 16) & 0xFF,
+                (pci_dev->class >> 8) & 0xFF,
+                pci_dev->vendor,
+                pci_dev->device,
+                pci_dev->subsystem_vendor,
+                pci_dev->subsystem_device,
+                NV_TRUE /* print_legacy_warning */))
         {
             count++;
-            nv_pci_validate_assigned_gpus(pci_dev);
-        }
-        pci_dev = NV_PCI_GET_CLASS(PCI_CLASS_DISPLAY_3D << 8, pci_dev);
-    }
-
-    if (NV_IS_ASSIGN_GPU_PCI_INFO_SPECIFIED())
-    {
-        NvU32 i;
-
-        for (i = 0; i < nv_assign_gpu_count; i++)
-        {
-            if (nv_assign_gpu_pci_info[i].valid == NV_TRUE)
-                count++;
         }
+        pci_dev = pci_get_class(PCI_CLASS_DISPLAY_3D << 8, pci_dev);
     }
 
     return count;
