@@ -48,7 +48,6 @@ static struct proc_dir_entry *proc_nvidia_gpus;
 extern char *NVreg_RegistryDwords;
 extern char *NVreg_RegistryDwordsPerDevice;
 extern char *NVreg_RmMsg;
-extern char *NVreg_AssignGpus;
 extern char *NVreg_GpuBlacklist;
 extern char *NVreg_TemporaryFilePath;
 
@@ -61,7 +60,7 @@ static nv_pm_action_depth_t nv_pm_action_depth = NV_PM_ACTION_DEPTH_DEFAULT;
 static int nv_procfs_read_registry(struct seq_file *s, void *v);
 
 #define NV_NUMA_STATUS_MSG_LEN      (32)
-#define NV_PROC_WRITE_BUFFER_SIZE   (64 * PAGE_SIZE)
+#define NV_PROC_WRITE_BUFFER_SIZE   (128 * PAGE_SIZE)
 
 /*
  * Status messages directly corresponding to states in nv_numa_states_t.
@@ -120,8 +119,8 @@ nv_procfs_read_gpu_info(
     }
 
     if (rm_get_vbios_version(sp, nv, &vbios_rev1, &vbios_rev2,
-                &vbios_rev3, &vbios_rev4,
-                &vbios_rev5) != NV_OK)
+                             &vbios_rev3, &vbios_rev4,
+                             &vbios_rev5) != NV_OK)
     {
         seq_printf(s, "Video BIOS: \t ??.??.??.??.??\n");
     }
@@ -159,7 +158,7 @@ nv_procfs_read_gpu_info(
 #endif
 
     seq_printf(s, "Blacklisted:\t %s\n",
-               ((nv->flags & NV_FLAG_BLACKLIST) != 0) ? "Yes" : "No");
+               ((nv->flags & NV_FLAG_EXCLUDE) != 0) ? "Yes" : "No");
 
     rm_unref_dynamic_power(sp, nv, NV_DYNAMIC_PM_COARSE);
 
@@ -417,8 +416,6 @@ nv_procfs_read_params(
                (NVreg_RegistryDwordsPerDevice != NULL) ? NVreg_RegistryDwordsPerDevice : "");
     seq_printf(s, "RmMsg: \"%s\"\n",
                (NVreg_RmMsg != NULL) ? NVreg_RmMsg : "");
-    seq_printf(s, "AssignGpus: \"%s\"\n",
-               (NVreg_AssignGpus != NULL) ? NVreg_AssignGpus : "");
     seq_printf(s, "GpuBlacklist: \"%s\"\n",
                (NVreg_GpuBlacklist != NULL) ? NVreg_GpuBlacklist : "");
     seq_printf(s, "TemporaryFilePath: \"%s\"\n",
@@ -962,7 +959,7 @@ numa_status_read(
         NvS32 *status,
         NvU64 *numa_mem_addr,
         NvU64 *numa_mem_size,
-        nv_blacklist_addresses_t *blacklist
+        nv_offline_addresses_t *list
 )
 {
     NV_STATUS rm_status;
@@ -972,8 +969,8 @@ numa_status_read(
 
     /*
      * If GPU has not been initialized but NUMA info is valid, populate
-     * NUMA node ID and status. Memory range and blacklist cannot be read
-     * at this point so fill in dummy values.
+     * NUMA node ID and status. Memory range and offline addresses cannot
+     * be read at this point so fill in dummy values.
      */
     if (!(nv->flags & NV_FLAG_OPEN))
     {
@@ -984,7 +981,7 @@ numa_status_read(
             *status = nv_get_numa_status(nvl);
             *numa_mem_addr = 0;
             *numa_mem_size = 0;
-            memset(blacklist, 0x0, sizeof(*blacklist));
+            memset(list, 0x0, sizeof(*list));
         }
 #endif
 
@@ -992,8 +989,11 @@ numa_status_read(
         goto done;
     }
 
+    list->numEntries = ARRAY_SIZE(list->addresses);
+
     rm_status = rm_get_gpu_numa_info(sp, nv,
-                                     nid, numa_mem_addr, numa_mem_size, blacklist);
+                                     nid, numa_mem_addr, numa_mem_size,
+                                     list->addresses, &list->numEntries);
     *status = nv_get_numa_status(nvl);
 
 done:
@@ -1020,15 +1020,15 @@ nv_procfs_read_offline_pages(
                                  &numa_info.status,
                                  &numa_info.numa_mem_addr,
                                  &numa_info.numa_mem_size,
-                                 &numa_info.blacklist_addresses);
+                                 &numa_info.offline_addresses);
 
     if (rm_status != NV_OK)
         return -EIO;
 
-    for (i = 0; i < numa_info.blacklist_addresses.numEntries; ++i)
+    for (i = 0; i < numa_info.offline_addresses.numEntries; ++i)
     {
         seq_printf(s, "%p\n",
-                (void*) numa_info.blacklist_addresses.addresses[i]);
+                (void*) numa_info.offline_addresses.addresses[i]);
     }
 
     return retval;
@@ -1102,7 +1102,7 @@ nv_procfs_read_numa_status(
                                  &numa_info.status,
                                  &numa_info.numa_mem_addr,
                                  &numa_info.numa_mem_size,
-                                 &numa_info.blacklist_addresses);
+                                 &numa_info.offline_addresses);
 
     if ((rm_status != NV_OK) && (rm_status != NV_ERR_NOT_READY))
         return -EIO;
@@ -1303,7 +1303,7 @@ void nv_procfs_add_warning(
 #endif
 }
 
-int nv_register_procfs(void)
+int nv_procfs_init(void)
 {
 #if defined(CONFIG_PROC_FS)
     NvU32 i = 0;
@@ -1370,7 +1370,7 @@ failed:
 #endif
 }
 
-void nv_unregister_procfs(void)
+void nv_procfs_exit(void)
 {
 #if defined(CONFIG_PROC_FS)
     nv_procfs_unregister_all(proc_nvidia, proc_nvidia);
