@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1999-2013 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -25,7 +25,6 @@ static inline void nv_set_contig_memory_uc(nvidia_pte_t *page_ptr, NvU32 num_pag
 #endif
 }
 
-
 static inline void nv_set_contig_memory_wb(nvidia_pte_t *page_ptr, NvU32 num_pages)
 {
 #if defined(NV_SET_MEMORY_UC_PRESENT)
@@ -148,18 +147,20 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
 #else
     unsigned long *pages = NULL;
 #endif
+
     nvidia_pte_t *page_ptr;
     struct page *page;
 
     if (nv_set_memory_array_type_present(type))
     {
         status = os_alloc_mem((void **)&pages,
-                              at->num_pages * sizeof(unsigned long));
+                at->num_pages * sizeof(unsigned long));
+
     }
     else if (nv_set_pages_array_type_present(type))
     {
         status = os_alloc_mem((void **)&pages,
-                              at->num_pages * sizeof(struct page*));
+                at->num_pages * sizeof(struct page*));
     }
 
     if (status != NV_OK)
@@ -168,7 +169,7 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
     //
     // If the set_{memory,page}_array_* functions are in the kernel interface,
     // it's faster to use them since they work on non-contiguous memory,
-    // whereas the set_{memory,page}_* functions do not.
+    // whereas the set_{memory,page}_*  functions do not.
     //
     if (pages)
     {
@@ -189,6 +190,7 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
 #endif
         os_free_mem(pages);
     }
+
     //
     // If the set_{memory,page}_array_* functions aren't present in the kernel
     // interface, each page has to be set individually, which has been measured
@@ -201,72 +203,6 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
     }
 }
 
-/*
- * Cache flushes and TLB invalidation
- *
- * Allocating new pages, we may change their kernel mappings' memory types
- * from cached to UC to avoid cache aliasing. One problem with this is
- * that cache lines may still contain data from these pages and there may
- * be then stale TLB entries.
- *
- * Linux kernels prior to 2.6.25 fail to reliably flush caches on all or
- * some CPUS. The NVIDIA Linux graphics driver implements nv_flush_cache()
- * to perform heavy-weight flush/invalidation operations for older kernels
- * to avoid problems due to stale cache lines and/or TLB entries.
- *
- * Linux kernels which include tlbflush.h reliably flush caches when the
- * NVIDIA Linux graphics driver calls change_page_attr(...) after allocating
- * memory.
- */
-
-#if defined(NV_XEN_SUPPORT_FULLY_VIRTUALIZED_KERNEL) || \
-  defined(NV_CONFIG_PREEMPT_RT) || defined(NV_ASM_TLBFLUSH_H_PRESENT)
-#define NV_REQUIRE_HEAVY_WEIGHT_FLUSH 0
-#else
-#define NV_REQUIRE_HEAVY_WEIGHT_FLUSH 1
-#endif
-
-#if NV_REQUIRE_HEAVY_WEIGHT_FLUSH
-static void nv_flush_cache(void *p)
-{
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
-    unsigned long reg0, reg1, reg2;
-#endif
-
-    CACHE_FLUSH();
-
-    // flush global TLBs
-#if defined(NVCPU_X86)
-    asm volatile("movl %%cr4, %0;  \n"
-                 "movl %0, %2;     \n"
-                 "andl $~0x80, %0; \n"
-                 "movl %0, %%cr4;  \n"
-                 "movl %%cr3, %1;  \n"
-                 "movl %1, %%cr3;  \n"
-                 "movl %2, %%cr4;  \n"
-                 : "=&r" (reg0), "=&r" (reg1), "=&r" (reg2)
-                 : : "memory");
-#elif defined(NVCPU_X86_64)
-    asm volatile("movq %%cr4, %0;  \n"
-                 "movq %0, %2;     \n"
-                 "andq $~0x80, %0; \n"
-                 "movq %0, %%cr4;  \n"
-                 "movq %%cr3, %1;  \n"
-                 "movq %1, %%cr3;  \n"
-                 "movq %2, %%cr4;  \n"
-                 : "=&r" (reg0), "=&r" (reg1), "=&r" (reg2)
-                 : : "memory");
-#endif
-}
-#endif
-
-static inline void nv_flush_caches(void)
-{
-#if NV_REQUIRE_HEAVY_WEIGHT_FLUSH
-    on_each_cpu(nv_flush_cache, NULL, 1);
-#endif
-}
-
 static NvU64 nv_get_max_sysmem_address(void)
 {
     NvU64 global_max_pfn = 0ULL;
@@ -377,7 +313,6 @@ static NV_STATUS nv_alloc_coherent_pages(
         nv_set_contig_memory_type(at->page_table[0],
                                   at->num_pages,
                                   NV_MEMORY_UNCACHED);
-        nv_flush_caches();
     }
 
     return NV_OK;
@@ -401,9 +336,6 @@ static void nv_free_coherent_pages(
 
     dma_free_coherent(dev, at->num_pages * PAGE_SIZE,
                       (void *)page_ptr->virt_addr, page_ptr->dma_addr);
-
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_flush_caches();
 }
 
 NV_STATUS nv_alloc_contig_pages(
@@ -475,9 +407,6 @@ NV_STATUS nv_alloc_contig_pages(
                                   NV_MEMORY_UNCACHED);
     }
 
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_flush_caches();
-
     return NV_OK;
 
 failed:
@@ -531,9 +460,6 @@ void nv_free_contig_pages(
         NV_MAYBE_UNRESERVE_PAGE(page_ptr);
     }
 
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_flush_caches();
-
     page_ptr = at->page_table[0];
 
     NV_FREE_PAGES(page_ptr->virt_addr, at->order);
@@ -618,9 +544,6 @@ NV_STATUS nv_alloc_system_pages(
     if (at->cache_type != NV_MEMORY_CACHED)
         nv_set_memory_type(at, NV_MEMORY_UNCACHED);
 
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_flush_caches();
-
     return NV_OK;
 
 failed:
@@ -668,9 +591,6 @@ void nv_free_system_pages(
         NV_MAYBE_UNRESERVE_PAGE(page_ptr);
         NV_FREE_PAGES(page_ptr->virt_addr, 0);
     }
-
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_flush_caches();
 }
 
 NvUPtr nv_vm_map_pages(
