@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2013-2019 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2013-2020 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -1230,26 +1230,6 @@ NV_STATUS nvUvmInterfaceRetainChannel(uvmGpuAddressSpaceHandle vaSpace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceRetainChannel);
 
-NV_STATUS nvUvmInterfaceRetainChannelResources(void *retainedChannel,
-                                               UvmGpuChannelResourceInfo *channelResourceInfo)
-{
-    nvidia_stack_t *sp = NULL;
-    NV_STATUS status;
-
-    if (nv_kmem_cache_alloc_stack(&sp) != 0)
-    {
-        return NV_ERR_NO_MEMORY;
-    }
-
-    status = rm_gpu_ops_retain_channel_resources(sp,
-                                                 retainedChannel,
-                                                 channelResourceInfo);
-
-    nv_kmem_cache_free_stack(sp);
-    return status;
-}
-EXPORT_SYMBOL(nvUvmInterfaceRetainChannelResources);
-
 NV_STATUS nvUvmInterfaceBindChannelResources(void *retainedChannel,
                                              UvmGpuChannelResourceBindParams *channelResourceBindParams)
 {
@@ -1280,16 +1260,6 @@ void nvUvmInterfaceReleaseChannel(void *retainedChannel)
 }
 EXPORT_SYMBOL(nvUvmInterfaceReleaseChannel);
 
-void nvUvmInterfaceReleaseChannelResources(NvP64 *resourceDescriptors, NvU32 descriptorCount)
-{
-    nvidia_stack_t *sp = nvUvmGetSafeStack();
-
-    rm_gpu_ops_release_channel_resources(sp, resourceDescriptors, descriptorCount);
-
-    nvUvmFreeSafeStack(sp);
-}
-EXPORT_SYMBOL(nvUvmInterfaceReleaseChannelResources);
-
 void nvUvmInterfaceStopChannel(void *retainedChannel, NvBool bImmediate)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
