@@ -872,7 +872,9 @@ void* NV_API_CALL os_map_kernel_space(
             vaddr = nv_ioremap_cache(start, size_bytes);
             break;
         case NV_MEMORY_WRITECOMBINED:
-            vaddr = nv_ioremap_wc(start, size_bytes);
+            vaddr = rm_disable_iomap_wc() ? 
+                    nv_ioremap_nocache(start, size_bytes) :
+                    nv_ioremap_wc(start, size_bytes);
             break;
         case NV_MEMORY_UNCACHED:
         case NV_MEMORY_DEFAULT:
@@ -1121,11 +1123,6 @@ NvBool NV_API_CALL os_is_efi_enabled(void)
     return NV_EFI_ENABLED();
 }
 
-NvBool NV_API_CALL os_iommu_is_snooping_enabled(void)
-{
-    return NV_TRUE;
-}
-
 void NV_API_CALL os_get_screen_info(
     NvU64 *pPhysicalAddress,
     NvU16 *pFbWidth,
@@ -1717,10 +1714,20 @@ NV_STATUS NV_API_CALL os_open_temporary_file
     void **ppFile
 )
 {
-#if defined(O_TMPFILE) && defined(NV_KERNEL_WRITE_PRESENT)
+#if defined(O_TMPFILE)
     struct file *file;
     const char *path = NVreg_TemporaryFilePath;
 
+    /*
+     * The filp_open() call below depends on the current task's fs_struct
+     * (current->fs), which may already be NULL if this is called during
+     * process teardown.
+     */
+    if (current->fs == NULL)
+    {
+        return NV_ERR_OPERATING_SYSTEM;
+    }
+
     if (!path)
         path = "/tmp";
 
@@ -1743,9 +1750,7 @@ void NV_API_CALL os_close_file
     void *pFile
 )
 {
-#if defined(O_TMPFILE) && defined(NV_KERNEL_WRITE_PRESENT)
     filp_close(pFile, NULL);
-#endif
 }
 
 #define NV_MAX_NUM_FILE_IO_RETRIES 10
@@ -1758,7 +1763,7 @@ NV_STATUS NV_API_CALL os_write_file
     NvU64 offset
 )
 {
-#if defined(O_TMPFILE) && defined(NV_KERNEL_WRITE_PRESENT)
+#if defined(NV_KERNEL_WRITE_PRESENT)
     loff_t f_pos = offset;
     ssize_t num_written;
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
@@ -1802,7 +1807,6 @@ NV_STATUS NV_API_CALL os_read_file
     NvU64 offset
 )
 {
-#if defined(O_TMPFILE) && defined(NV_KERNEL_READ_PRESENT)
     loff_t f_pos = offset;
     ssize_t num_read;
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
@@ -1833,9 +1837,58 @@ retry:
     }
 
     return NV_OK;
-#else
-    return NV_ERR_NOT_SUPPORTED;
-#endif
+}
+
+NV_STATUS NV_API_CALL os_open_readonly_file
+(
+    const char  *filename,
+    void       **ppFile
+)
+{
+    struct file *file;
+
+    /*
+     * The filp_open() call below depends on the current task's fs_struct
+     * (current->fs), which may already be NULL if this is called during
+     * process teardown.
+     */
+    if (current->fs == NULL)
+    {
+        return NV_ERR_OPERATING_SYSTEM;
+    }
+
+    file = filp_open(filename, O_RDONLY, 0);
+    if (IS_ERR(file))
+    {
+        return NV_ERR_OPERATING_SYSTEM;
+    }
+
+    *ppFile = (void *)file;
+
+    return NV_OK;
+}
+
+NV_STATUS NV_API_CALL os_open_and_read_file
+(
+    const char *filename,
+    NvU8       *buf,
+    NvU64       count
+)
+{
+    void *fileHandle;
+    NV_STATUS status;
+
+    status = os_open_readonly_file(filename, &fileHandle);
+    if (status != NV_OK)
+    {
+        return status;
+    }
+
+    status = os_read_file(fileHandle, buf, count, 0);
+
+    os_close_file(fileHandle);
+
+    return status;
 }
 
 
